/////////////////////////////////////////////////////////////////////////////
// Name:        fiber.cpp
// Purpose:     Generate TAM tilesets which self-assemble into fibered fractals
// Author:      Matthew Patitz <mpatitz@yahoo.com>
// Modified by:
// Created:     Nov/2007
// Copyright:   (c) 2008, Matthew John Patitz
// Licence:     GPL
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "points.h"
#include "fiber.h"


std::string GetPrefixFromNum(int num, int iDirGrow, PointSet* pPS)
{
  std::string sOrient = "H";
  if ((iDirGrow == NORTH) || (iDirGrow == SOUTH)) {
    sOrient = "V";
  } else if (iDirGrow == -1) {
    sOrient = "T";
  }

  std::string sRet;
  if (num >= 0) {
    sRet = std::string("{(") + itos(pPS->at(num).GetX()) + "," + itos(pPS->at(num).GetY()) + ")" + sOrient + "}";
  } else {
    sRet = std::string("{") + itos(num) + sOrient + "}";
  }
  return sRet;
}

int GetReversedPosition(int pos, int base)
{
  return (base - pos);
}

std::string GenerateTileName(Point pt, int iSetNum, bool bWithPrefix)
{
  std::string prefix = "";
  if (bWithPrefix) {
    prefix = std::string("[") + itos(iSetNum) + std::string("]");
  }
  return prefix + itos(pt.GetX()) + "," + itos(pt.GetY());
}

int ConvertLocationToType(int i, int iDirGrow, PointSet* pPS)
{
  int iType = -1;
  if ((iDirGrow == NORTH) || (iDirGrow == SOUTH)) {
    iType = pPS->GetPointNumForLocation(Point(0, i));
  } else if ((iDirGrow == EAST) || (iDirGrow == WEST)) {
    iType = pPS->GetPointNumForLocation(Point(i, 0));
  }
  return iType;
}

std::string BraceNumber(int i, int iDirGrow, PointSet* pPS)
{
  int iType = ConvertLocationToType(i, iDirGrow, pPS);
  return std::string("[") + itos(iType) + "]";
}

int TranslateDirection(int iDir, int iDirGrow, int iDirFace)
{
  if ((iDirGrow == NORTH) && (iDirFace == EAST)) {
    return iDir;
  } else if ((iDirGrow == EAST) && (iDirFace == NORTH)) {
    if ((iDir == NORTH) || (iDir == SOUTH)) {
      return iDir + 1;
    } else {
      return iDir - 1;
    }
  } else {
    return -1;
  }
}

TileSet CreateCounterTileSet(int iBase, int iType, int iOrientation, PointSet* pPS)
{
  TileSet ts;

  Point* ptThis  = &pPS->at(iType);
  Point* ptInput = ptThis->GetInput();
  int dirInput = SOUTH;
  if (ptInput) {
    dirInput = ptThis->GetInputDir();
  }

  int dirForward  = NORTH;
  int dirReverse  = SOUTH;
  int dirFace     = EAST;
  int dirBack     = WEST;

  if (iOrientation == HORIZONTAL) {
    dirFace       = NORTH;
    dirBack       = SOUTH;
    if (dirInput == EAST) {
      dirForward  = WEST;
      dirReverse  = EAST;
    } else {
      dirForward  = EAST;
      dirReverse  = WEST;
    }
  } else {
    if (dirInput == NORTH) {
      dirForward  = SOUTH;
      dirReverse  = NORTH;
    }
  }

  int strengthForward = 0;
  if (pPS->GetPointNumForLocation(ptThis->GetOffset(dirForward, 1)) > -1) {
    strengthForward = 2;
  }

  bool bReverseGrows = false;
  if ((dirInput == EAST) || (dirInput == NORTH)) {
    bReverseGrows = true;
  }

  std::string strMostsigdirFace  = MOSTSIGNOGROW;

  Point* ptOut = pPS->at(iType).GetOutput(dirForward);


  /////////////////////////////////////////////////////////////////////////////
  //  Tiles for single-bit width copy of counter (not including spacing rows!)
  for (int i=1; i<iBase; i++) {
    Tile f;
    f.UsePrefix(true);

    f.m_sides[dirForward].m_strength = 2;
    f.m_sides[dirForward].m_label    = std::string(MOSTSIG) + itos(i) + LEASTSIG;

    //  Special case for Sierpinski Triangle due to small base 2 wraparound
    if ((i == (iBase-1)) && (iBase == 2) && (iType != 0)) {
      f.m_sides[dirForward].m_label  = std::string(GROWFIBERMARKER) + f.m_sides[dirForward].m_label;
    }

    f.m_sides[dirReverse].m_strength = 2;
    f.m_sides[dirReverse].m_label    = std::string(MOSTSIG) + itos(i-1) + LEASTSIG;

    f.m_sides[dirBack].m_strength  = 0;
    f.m_sides[dirBack].m_label     = NODISPLAY;

    f.m_sides[dirFace].m_bAddPrefix= false;
    Point pt;
    int pn = -1;
    if (iOrientation == VERTICAL) {
      int pi = i;
      if (dirForward == SOUTH) {
        pi = GetReversedPosition(i, iBase);
      }
      pn = pPS->GetPointNumForLocation(Point(1, pi));
      pt = Point(0, pi);
    } else {
      int pi = i;
      if (dirForward == WEST) {
        pi = GetReversedPosition(i, iBase);
      }
      pn = pPS->GetPointNumForLocation(Point(pi, 1));
      pt = Point(pi, 0);
    }
    if (pn != -1) {
      f.m_sides[dirFace].m_strength  = 2;
      f.m_sides[dirFace].m_label     = itos(pt.GetX()) + "," + itos(pt.GetY());
    } else {
      f.m_sides[dirFace].m_strength  = 0;
      f.m_sides[dirFace].m_label     = NODISPLAY;
    }

    if (i == iBase-1) {
      if (!ptOut && (!bReverseGrows || (dirInput == iOrientation))) {  //  Trick to 'cap off' dead end from reverse growth
        f.m_sides[dirForward].m_bAddPrefix = false;
        f.m_sides[dirForward].m_strength   = 0;
        f.m_sides[dirForward].m_label      = NODISPLAY;
      }
    }

    ts.push_back(f);

    //  Copy for counters that shouldn't grow MSB's
    f.m_sides[dirForward].m_strength = 2;
    f.m_sides[dirForward].m_label    = std::string(MOSTSIGNOGROW) + itos(i) + LEASTSIG;

    f.m_sides[dirReverse].m_strength = 2;
    f.m_sides[dirReverse].m_label    = std::string(MOSTSIGNOGROW) + itos(i-1) + LEASTSIG;

    if (i == iBase-1) {
      Point* ptOut = pPS->at(iType).GetOutput(dirForward);
      if (!ptOut && (!bReverseGrows || (dirInput == iOrientation))) {  //  Trick to 'cap off' dead end from reverse growth
        f.m_sides[dirForward].m_bAddPrefix = false;
        f.m_sides[dirForward].m_strength   = 0;
        f.m_sides[dirForward].m_label      = NODISPLAY;
      }
    }

    ts.push_back(f);

  }

  //  Optimization:  bail out here for type 0 because no more tiletypes are needed for it
  if (iType == 0) {
    ts.SetPrefix(GetPrefixFromNum(iType, dirForward, pPS));
    return ts;
  }

  ///////////////////////////////////////////////////
  //  Tiles for multi-bit width copies of counter

  //  Tiles in LSB position:
  //    1. with marker and value (iBase-1) and passing MARKER left
  //    2. with marker value != (iBase-1) and just incrementing
  for (int i=1; i<iBase; i++) {
    Tile g;

    std::string strMarker = MARKER;
    if (i == (iBase-1)) {
      strMarker = "";
      g.m_sides[dirForward].m_strength  = 1;
      g.m_sides[dirBack].m_label        = std::string(COPY) + MARKER;
    } else {
      g.m_sides[dirForward].m_strength  = 2;
      g.m_sides[dirBack].m_label        = std::string(COPY);
    }
    g.m_display                       = itos(i) + strMarker + LEASTSIG;
    g.m_sides[dirForward].m_label     = itos(i) + strMarker + LEASTSIG;

    //  Initiate growth of generic non-axes tiles if necessary
    g.m_sides[dirFace].m_bAddPrefix= false;
    Point pt;
    int pn = -1;
    if (iOrientation == VERTICAL) {
      int pi = i;
      if (dirForward == SOUTH) {
        pi = GetReversedPosition(i, iBase);
      }
      pn = pPS->GetPointNumForLocation(Point(1, pi));
      pt = Point(0, pi);
    } else {
      int pi = i;
      if (dirForward == WEST) {
        pi = GetReversedPosition(i, iBase);
      }
      pn = pPS->GetPointNumForLocation(Point(pi, 1));
      pt = Point(pi, 0);
    }
    if (pn != -1) {
      g.m_sides[dirFace].m_strength  = 2;
      g.m_sides[dirFace].m_label     = itos(pt.GetX()) + "," + itos(pt.GetY());
    } else {
      g.m_sides[dirFace].m_strength  = 0;
      g.m_sides[dirFace].m_label     = NODISPLAY;
    }

    g.m_sides[dirReverse].m_strength  = 2;
    g.m_sides[dirReverse].m_label     = itos(i-1) + MARKER + LEASTSIG;

    g.m_sides[dirBack].m_strength     = 1;

    ts.push_back(g);
  }

  //  Tiles in MSB position that receive the MARKER from the right
  for (int i=1; i<iBase; i++) {
    Tile a;
    a.m_display                       = std::string(MOSTSIG) + itos(i) + MARKER;
    a.m_sides[dirForward].m_strength  = 2;
    a.m_sides[dirFace].m_strength     = 1;
    a.m_sides[dirReverse].m_strength  = 1;
    a.m_sides[dirBack].m_strength     = 0;
    a.m_sides[dirForward].m_label     = std::string(MOSTSIG) + itos(i) + std::string(SPACERMARKER) + MARKER;
    a.m_sides[dirFace].m_label        = std::string(COPY) + MARKER;
    a.m_sides[dirReverse].m_label     = std::string(MOSTSIG) + itos(i);
    a.m_sides[dirBack].m_label        = NODISPLAY;
    ts.push_back(a);

    if (bReverseGrows) {
      if (!ptOut && (i == iBase-1) && (dirInput == iOrientation)) {     //  Trick to 'cap off' dead end from reverse growth
        a.m_sides[dirForward].m_strength  = 0;
        a.m_sides[dirForward].m_label     = NODISPLAY;
      } else {
        a.m_sides[dirForward].m_strength  = 2;
        a.m_sides[dirForward].m_label     = std::string(MOSTSIGNOGROW) + itos(i) + std::string(SPACERMARKER) + MARKER;
      }
    } else {
      a.m_sides[dirForward].m_strength  = ((i == (iBase-1)) ? strengthForward : 2);
      if ((i == (iBase-1)) && (strengthForward == 0)) {
        a.m_sides[dirForward].m_label   = NODISPLAY;
      } else {
        a.m_sides[dirForward].m_label   = std::string(MOSTSIGNOGROW) + itos(i) + std::string(SPACERMARKER) + MARKER;
      }
    }
    a.m_sides[dirReverse].m_label     = std::string(MOSTSIGNOGROW) + itos(i);
    ts.push_back(a);
  }

  //  Tiles in MSB position that increment and pass the MARKER back to the right
  for (int i=1; i<iBase; i++) {
    Tile a;
    a.m_display                       = std::string(MOSTSIG) + itos(i);
    a.m_sides[dirForward].m_strength  = 1;
    a.m_sides[dirFace].m_strength     = 1;
    a.m_sides[dirReverse].m_strength  = 2;
    a.m_sides[dirBack].m_strength     = 0;
    a.m_sides[dirForward].m_label     = std::string(MOSTSIG) + itos(i);
    a.m_sides[dirFace].m_label        = MARKER;
    a.m_sides[dirReverse].m_label     = std::string(MOSTSIG) + itos(i-1) + MARKER;
    a.m_sides[dirBack].m_label        = NODISPLAY;
    ts.push_back(a);

    a.m_sides[dirForward].m_label     = std::string(MOSTSIGNOGROW) + itos(i);
    a.m_sides[dirReverse].m_label     = std::string(MOSTSIGNOGROW) + itos(i-1) + MARKER;
    ts.push_back(a);
  }

  //  MSB tile in fixed-width counter with value 0 that gets MARKER from the right
  Tile d;
  d.m_display                         = std::string(MOSTSIG) + itos(0) + MARKER;
  d.m_sides[dirForward].m_strength    = 2;
  d.m_sides[dirFace].m_strength       = 1;
  d.m_sides[dirReverse].m_strength    = 1;
  d.m_sides[dirBack].m_strength       = 0;
  d.m_sides[dirForward].m_label       = std::string(MOSTSIG) + itos(0) + std::string(SPACERMARKER) + MARKER;
  d.m_sides[dirFace].m_label          = std::string(COPY) + std::string(MARKER);
  d.m_sides[dirReverse].m_label       = std::string(MOSTSIG) + itos(0);
  d.m_sides[dirBack].m_label          = NODISPLAY;
  ts.push_back(d);

  d.m_sides[dirForward].m_label       = std::string(MOSTSIGNOGROW) + itos(0) + std::string(SPACERMARKER) + MARKER;
  d.m_sides[dirReverse].m_label       = std::string(MOSTSIGNOGROW) + itos(0);
  ts.push_back(d);

  //  MSB tile in fixed-width counter that flips back to 0
  Tile e;
  e.m_display                         = std::string(MOSTSIG) + itos(0);
  e.m_sides[dirForward].m_strength    = 1;
  e.m_sides[dirFace].m_strength       = 1;
  e.m_sides[dirReverse].m_strength    = 2;
  e.m_sides[dirBack].m_strength       = 0;
  e.m_sides[dirForward].m_label       = std::string(MOSTSIG) + itos(0);
  e.m_sides[dirFace].m_label          = std::string(MARKER);
  e.m_sides[dirReverse].m_label       = std::string(MOSTSIG) + itos(iBase-1) + MARKER;
  e.m_sides[dirBack].m_label          = NODISPLAY;
  ts.push_back(e);

  e.m_sides[dirForward].m_label       = std::string(MOSTSIGNOGROW) + itos(0);
  e.m_sides[dirReverse].m_label       = std::string(MOSTSIGNOGROW) + itos(iBase-1) + MARKER;
  ts.push_back(e);

  //  Tiles in LSB position that receive the MARKER from the left
  for (int i=0; i<iBase; i++) {
    Tile a;
    a.m_display                       = itos(0) + MARKER + LEASTSIG;
    a.m_sides[dirForward].m_strength  = 2;
    a.m_sides[dirFace].m_strength     = 0;
    a.m_sides[dirReverse].m_strength  = 1;
    a.m_sides[dirBack].m_strength     = 1;
    a.m_sides[dirForward].m_label     = itos(0) + MARKER + LEASTSIG;
    a.m_sides[dirFace].m_label        = NODISPLAY;
    a.m_sides[dirReverse].m_label     = itos(i) + LEASTSIG;
    a.m_sides[dirBack].m_label        = MARKER;
    ts.push_back(a);
  }

  //  Tiles in interior positions that receive the MARKER from the right
  //  This is where spacing rows are initiated
  for (int i=0; i<iBase-1; i++) {
    Tile a;
    a.m_display                       = itos(i) + MARKER;
    a.m_sides[dirForward].m_strength  = 2;
    a.m_sides[dirFace].m_strength     = 1;
    a.m_sides[dirReverse].m_strength  = 1;
    a.m_sides[dirBack].m_strength     = 1;
    a.m_sides[dirForward].m_label   = itos(i) + std::string(SPACERMARKER) + MARKER;
    a.m_sides[dirFace].m_label        = std::string(COPY) + MARKER;
    a.m_sides[dirReverse].m_label     = itos(i);
    a.m_sides[dirBack].m_label        = COPY;
    ts.push_back(a);
  }

  //  Tiles in interior positions & MSB position that receive COPY from the right
  for (int i=0; i<iBase; i++) {
    Tile a;
    a.m_display                       = itos(i);
    a.m_sides[dirForward].m_strength  = 1;
    a.m_sides[dirFace].m_strength     = 1;
    a.m_sides[dirReverse].m_strength  = 1;
    a.m_sides[dirBack].m_strength     = 1;
    a.m_sides[dirForward].m_label     = itos(i);
    a.m_sides[dirFace].m_label        = COPY;
    a.m_sides[dirReverse].m_label     = itos(i);
    a.m_sides[dirBack].m_label        = COPY;
    ts.push_back(a);

    Tile b;
    b.m_display                       = MOSTSIG + itos(i);
    b.m_sides[dirForward].m_strength  = 1;
    b.m_sides[dirFace].m_strength     = 1;
    b.m_sides[dirReverse].m_strength  = 1;
    b.m_sides[dirBack].m_strength     = 0;
    b.m_sides[dirForward].m_label     = MOSTSIG + itos(i);
    b.m_sides[dirFace].m_label        = COPY;
    b.m_sides[dirReverse].m_label     = MOSTSIG + itos(i);
    b.m_sides[dirBack].m_label        = NODISPLAY;
    ts.push_back(b);

    b.m_sides[dirForward].m_label     = MOSTSIGNOGROW + itos(i);
    b.m_sides[dirReverse].m_label     = MOSTSIGNOGROW + itos(i);
    ts.push_back(b);
  }

  //  Tiles in interior positions that are zeroed and pass the MARKER back to the right
  for (int i=0; i<iBase; i++) {
    Tile a;
    a.m_display                       = itos(0);
    a.m_sides[dirForward].m_strength  = 1;
    a.m_sides[dirFace].m_strength     = 1;
    a.m_sides[dirReverse].m_strength  = 1;
    a.m_sides[dirBack].m_strength     = 1;
    a.m_sides[dirForward].m_label     = itos(0);
    a.m_sides[dirFace].m_label        = MARKER;
    a.m_sides[dirReverse].m_label     = itos(i);
    a.m_sides[dirBack].m_label        = MARKER;
    ts.push_back(a);
  }

  //  Tiles in interior positions that increment and pass the MARKER back to the right
  for (int i=0; i<iBase-1; i++) {
    Tile a;
    a.m_display                       = itos(i+1);
    a.m_sides[dirForward].m_strength  = 1;
    a.m_sides[dirFace].m_strength     = 1;
    a.m_sides[dirReverse].m_strength  = 2;
    a.m_sides[dirBack].m_strength     = 1;
    a.m_sides[dirForward].m_label     = itos(i+1);
    a.m_sides[dirFace].m_label        = MARKER;
    a.m_sides[dirReverse].m_label     = itos(i) + MARKER;
    a.m_sides[dirBack].m_label        = COPY;
    ts.push_back(a);
  }

  //  Tiles in interior positions and equal to (base-1) and which just continue passing the MARKER to the left
  Tile h;
  h.m_display                         = itos(iBase-1);
  h.m_sides[dirForward].m_strength    = 1;
  h.m_sides[dirFace].m_strength       = 1;
  h.m_sides[dirReverse].m_strength    = 1;
  h.m_sides[dirBack].m_strength       = 1;
  h.m_sides[dirForward].m_label       = itos(iBase-1);
  h.m_sides[dirFace].m_label          = std::string(COPY) + MARKER;
  h.m_sides[dirReverse].m_label       = itos(iBase-1);
  h.m_sides[dirBack].m_label          = std::string(COPY) + MARKER;
  ts.push_back(h);


  ////////////////////////////////
  //  Tiles for spacer blocks

  //  Leftmost tiles of a spacer block

  for (int i=0; i<iBase-1; i++) {
    //  Initiating row not in MSB
    Tile s;
    s.m_color                         = SPACERCOLOR;
    s.m_display                       = itos(i) + std::string(SPACERMARKER) + MARKER;
    s.m_sides[dirForward].m_strength  = 1;
    s.m_sides[dirFace].m_strength     = 1;
    s.m_sides[dirReverse].m_strength  = 2;
    s.m_sides[dirBack].m_strength     = 1;
    s.m_sides[dirForward].m_label     = itos(i+1);
    s.m_sides[dirFace].m_label        = SPACERMARKER + BraceNumber(i+1, dirForward, pPS) + std::string("1");
    s.m_sides[dirReverse].m_label     = itos(i) + std::string(SPACERMARKER) + MARKER;
    s.m_sides[dirBack].m_label        = SPACERCOPYLEFT + std::string(MOSTSIG);
    ts.push_back(s);

    //  Initiating row in MSB
    s.m_color                         = SPACERCOLOR;
    s.m_display                       = MOSTSIG + itos(i) + std::string(SPACERMARKER) + MARKER;
    s.m_sides[dirForward].m_strength  = 1;
    s.m_sides[dirFace].m_strength     = 1;
    s.m_sides[dirReverse].m_strength  = 2;
    s.m_sides[dirBack].m_strength     = 1;
    s.m_sides[dirForward].m_label     = MOSTSIG + itos(i+1);
    s.m_sides[dirFace].m_label        = SPACERMARKER + BraceNumber(i+1, dirForward, pPS) + std::string("1");
    s.m_sides[dirReverse].m_label     = MOSTSIG + itos(i) + std::string(SPACERMARKER) + MARKER;
    s.m_sides[dirBack].m_label        = SPACERCOPYLEFT + std::string(MOSTSIG);
    ts.push_back(s);

    s.m_sides[dirForward].m_label     = MOSTSIGNOGROW + itos(i+1);
    s.m_sides[dirReverse].m_label     = MOSTSIGNOGROW + itos(i) + std::string(SPACERMARKER) + MARKER;
    ts.push_back(s);
  }

  for (int i=0; i<iBase; i++) {
    //  Leftmost tiles of a spacer block
    Tile s;
    s.m_color                         = SPACERCOLOR;
    s.m_display                       = MOSTSIG + itos(i);
    s.m_sides[dirForward].m_strength  = 1;
    s.m_sides[dirFace].m_strength     = 1;
    s.m_sides[dirReverse].m_strength  = 1;
    s.m_sides[dirBack].m_strength     = 1;
    s.m_sides[dirForward].m_label     = MOSTSIG + itos(i);;
    s.m_sides[dirFace].m_label        = std::string(SPACERCOPYLEFT) + MOSTSIG;
    s.m_sides[dirReverse].m_label     = MOSTSIG + itos(i);;
    s.m_sides[dirBack].m_label        = TOBEDETERMINED;
    ts.push_back(s);

    s.m_color                         = SPACERCOLOR;
    s.m_display                       = MOSTSIGNOGROW + itos(i);
    s.m_sides[dirForward].m_strength  = 1;
    s.m_sides[dirFace].m_strength     = 1;
    s.m_sides[dirReverse].m_strength  = 1;
    s.m_sides[dirBack].m_strength     = 1;
    s.m_sides[dirForward].m_label     = MOSTSIGNOGROW + itos(i);
    s.m_sides[dirFace].m_label        = std::string(SPACERCOPYLEFT) + MOSTSIG;
    s.m_sides[dirReverse].m_label     = MOSTSIGNOGROW + itos(i);
    s.m_sides[dirBack].m_label        = TOBEDETERMINED;
    ts.push_back(s);

    s.m_color                         = SPACERCOLOR;
    s.m_display                       = MOSTSIG + itos(i);
    s.m_sides[dirForward].m_strength  = 1;
    s.m_sides[dirFace].m_strength     = 1;
    s.m_sides[dirReverse].m_strength  = 1;
    s.m_sides[dirBack].m_strength     = 1;
    s.m_sides[dirForward].m_label     = MOSTSIG + itos(i);
    s.m_sides[dirFace].m_label        = std::string(SPACERCOPYLEFT) + LEASTSIG;
    s.m_sides[dirReverse].m_label     = MOSTSIG + itos(i);
    s.m_sides[dirBack].m_label        = TOBEDETERMINED;
    ts.push_back(s);

    s.m_color                         = SPACERCOLOR;
    s.m_display                       = MOSTSIGNOGROW + itos(i);
    s.m_sides[dirForward].m_strength  = 1;
    s.m_sides[dirFace].m_strength     = 1;
    s.m_sides[dirReverse].m_strength  = 1;
    s.m_sides[dirBack].m_strength     = 1;
    s.m_sides[dirForward].m_label     = MOSTSIGNOGROW + itos(i);
    s.m_sides[dirFace].m_label        = std::string(SPACERCOPYLEFT) + LEASTSIG;
    s.m_sides[dirReverse].m_label     = MOSTSIGNOGROW + itos(i);
    s.m_sides[dirBack].m_label        = TOBEDETERMINED;
    ts.push_back(s);

    s.m_color                         = SPACERCOLOR;
    s.m_display                       = MOSTSIG + itos(i);
    s.m_sides[dirForward].m_strength  = 1;
    s.m_sides[dirFace].m_strength     = 1;
    s.m_sides[dirReverse].m_strength  = 1;
    s.m_sides[dirBack].m_strength     = 1;
    s.m_sides[dirForward].m_label     = MOSTSIG + itos(i);;
    s.m_sides[dirFace].m_label        = std::string(SPACERCOPYLEFT);
    s.m_sides[dirReverse].m_label     = MOSTSIG + itos(i);;
    s.m_sides[dirBack].m_label        = TOBEDETERMINED;
    ts.push_back(s);

    s.m_color                         = SPACERCOLOR;
    s.m_display                       = MOSTSIGNOGROW + itos(i);
    s.m_sides[dirForward].m_strength  = 1;
    s.m_sides[dirFace].m_strength     = 1;
    s.m_sides[dirReverse].m_strength  = 1;
    s.m_sides[dirBack].m_strength     = 1;
    s.m_sides[dirForward].m_label     = MOSTSIGNOGROW + itos(i);
    s.m_sides[dirFace].m_label        = std::string(SPACERCOPYLEFT);
    s.m_sides[dirReverse].m_label     = MOSTSIGNOGROW + itos(i);
    s.m_sides[dirBack].m_label        = TOBEDETERMINED;
    ts.push_back(s);
  }

  for (int i=0; i<iBase; i++) {
    //  1. Spacer tiles in the bottom row and to the left of the SPACERMARKER
    //  2. Interior tiles in an interior row of a spacer block
    Tile s;
    s.m_color                         = SPACERCOLOR;
    s.m_display                       = itos(i);
    s.m_sides[dirForward].m_strength  = 1;
    s.m_sides[dirFace].m_strength     = 1;
    s.m_sides[dirReverse].m_strength  = 1;
    s.m_sides[dirBack].m_strength     = 1;
    s.m_sides[dirForward].m_label     = itos(i);
    s.m_sides[dirFace].m_label        = std::string(SPACERCOPYLEFT);
    s.m_sides[dirReverse].m_label     = itos(i);
    s.m_sides[dirBack].m_label        = std::string(SPACERCOPYLEFT);
    ts.push_back(s);

    s.m_color                         = SPACERCOLOR;
    s.m_display                       = itos(i);
    s.m_sides[dirForward].m_strength  = 1;
    s.m_sides[dirFace].m_strength     = 1;
    s.m_sides[dirReverse].m_strength  = 1;
    s.m_sides[dirBack].m_strength     = 1;
    s.m_sides[dirForward].m_label     = itos(i);
    s.m_sides[dirFace].m_label        = std::string(SPACERCOPYLEFT) + MOSTSIG;
    s.m_sides[dirReverse].m_label     = itos(i);
    s.m_sides[dirBack].m_label        = std::string(SPACERCOPYLEFT) + MOSTSIG;
    ts.push_back(s);
  }

  for (int i=0; i<iBase; i++) {
    //  The tile immediately to the right of the initial spacer tile and above an LSB
    Tile s;
    s.m_color                         = SPACERCOLOR;
    s.m_display                       = itos(iBase-1) + LEASTSIG;
    s.m_sides[dirForward].m_strength  = 2;
    s.m_sides[dirFace].m_strength     = 2;
    s.m_sides[dirReverse].m_strength  = 1;
    s.m_sides[dirBack].m_strength     = 1;
    s.m_sides[dirForward].m_label     = std::string("0") + std::string(MARKER) + LEASTSIG;
    s.m_sides[dirFace].m_bAddPrefix   = false;
    int pos = i;
    if ((dirForward == WEST) || (dirForward == SOUTH)) {
      pos = GetReversedPosition(i, iBase);
    }
    s.m_sides[dirFace].m_label        = GetPrefixFromNum(ConvertLocationToType(pos, dirForward, pPS), (dirForward+1)%4, pPS) + strMostsigdirFace + "0" + LEASTSIG;
    s.m_sides[dirReverse].m_label     = itos(iBase-1) + LEASTSIG;
    s.m_sides[dirBack].m_label        = std::string(SPACERMARKER) + BraceNumber(i, dirForward, pPS) + std::string("1");
    ts.push_back(s);

    //  The tile immediately to the right of the initial spacer tile but not above an LSB
    s.UsePrefix(true);
    s.m_color                         = SPACERCOLOR;
    s.m_sides[dirFace].m_bAddPrefix   = true;
    s.m_display                       = itos(iBase-1);
    s.m_sides[dirForward].m_strength  = 1;
    s.m_sides[dirFace].m_strength     = 1;
    s.m_sides[dirReverse].m_strength  = 1;
    s.m_sides[dirBack].m_strength     = 1;
    s.m_sides[dirForward].m_label     = std::string("0");
    s.m_sides[dirFace].m_label        = std::string(SPACERMARKER) + BraceNumber(i, dirForward, pPS);
    s.m_sides[dirReverse].m_label     = itos(iBase-1);
    s.m_sides[dirBack].m_label        = std::string(SPACERMARKER) + BraceNumber(i, dirForward, pPS) + std::string("1");
    ts.push_back(s);

    //  The tile immediately to the right of the above spacer tile
    s.m_color                         = SPACERCOLOR;
    s.m_display                       = itos(iBase-1);
    s.m_sides[dirForward].m_strength  = 2;
    s.m_sides[dirFace].m_strength     = 1;
    s.m_sides[dirReverse].m_strength  = 1;
    s.m_sides[dirBack].m_strength     = 1;
    s.m_sides[dirForward].m_label     = std::string(SPACERMARKER) + BraceNumber(i, dirForward, pPS);
    s.m_sides[dirFace].m_label        = std::string(MOSTSIG) + SPACERCOPYRIGHT + BraceNumber(i, dirForward, pPS);
    s.m_sides[dirReverse].m_label     = itos(iBase-1);
    s.m_sides[dirBack].m_label        = std::string(SPACERMARKER) + BraceNumber(i, dirForward, pPS);
    ts.push_back(s);

    //  Interior tiles in the bottom row of a spacer block and above a flipped bit
    s.m_color                         = SPACERCOLOR;
    s.m_display                       = itos(iBase-1);
    s.m_sides[dirForward].m_strength  = 1;
    s.m_sides[dirFace].m_strength     = 1;
    s.m_sides[dirReverse].m_strength  = 1;
    s.m_sides[dirBack].m_strength     = 1;
    s.m_sides[dirForward].m_label     = std::string("0");
    s.m_sides[dirFace].m_label        = MOSTSIG + std::string(SPACERCOPYRIGHT) + BraceNumber(i, dirForward, pPS);
    s.m_sides[dirReverse].m_label     = itos(iBase-1);
    s.m_sides[dirBack].m_label        = MOSTSIG + std::string(SPACERCOPYRIGHT) + BraceNumber(i, dirForward, pPS);
    ts.push_back(s);

    //  Interior tiles in the bottom row of a spacer block and above a flipped bit
    s.m_color                         = SPACERCOLOR;
    s.m_display                       = std::string("0") + LEASTSIG;
    s.m_sides[dirForward].m_strength  = 1;
    s.m_sides[dirFace].m_strength     = 1;
    s.m_sides[dirReverse].m_strength  = 1;
    s.m_sides[dirBack].m_strength     = 1;
    s.m_sides[dirForward].m_label     = std::string("0") + LEASTSIG;
    s.m_sides[dirFace].m_bAddPrefix   = false;
    pos = i;
    if ((dirForward == WEST) || (dirForward == SOUTH)) {
      pos = GetReversedPosition(i, iBase);
    }
    s.m_sides[dirFace].m_label        = GetPrefixFromNum(ConvertLocationToType(pos, dirForward, pPS), (dirForward+1)%4, pPS) + "0";
    s.m_sides[dirReverse].m_label     = std::string("0") + LEASTSIG;
    s.m_sides[dirBack].m_label        = std::string(SPACERCOPYRIGHT) + BraceNumber(i, dirForward, pPS);
    ts.push_back(s);

    //  Interior spacer tile above a flipped bit
    s.UsePrefix(true);
    s.m_color                         = SPACERCOLOR;
    s.m_display                       = std::string("0");
    s.m_sides[dirForward].m_strength  = 1;
    s.m_sides[dirFace].m_strength     = 1;
    s.m_sides[dirReverse].m_strength  = 1;
    s.m_sides[dirBack].m_strength     = 1;
    s.m_sides[dirForward].m_label     = std::string("0");
    s.m_sides[dirFace].m_label        = std::string(SPACERCOPYRIGHT) + BraceNumber(i, dirForward, pPS);
    s.m_sides[dirReverse].m_label     = std::string("0");
    s.m_sides[dirBack].m_label        = std::string(SPACERCOPYRIGHT) + BraceNumber(i, dirForward, pPS);
    ts.push_back(s);

    //  The rightmost tile of the first spacer row (as long as there were 2 bits flipped)
    s.UsePrefix(true);
    s.m_color                         = SPACERCOLOR;
    s.m_display                       = itos(iBase-1) + LEASTSIG;
    s.m_sides[dirForward].m_strength  = 2;
    s.m_sides[dirReverse].m_strength  = 1;
    s.m_sides[dirBack].m_strength     = 1;
    s.m_sides[dirForward].m_label     = std::string(SPACERMARKER) + "0" + LEASTSIG;
    s.m_sides[dirFace].m_bAddPrefix   = false;
    pos = i;
    if ((dirForward == WEST) || (dirForward == SOUTH)) {
      pos = GetReversedPosition(i, iBase);
      s.m_sides[dirFace].m_strength   = 2;
      s.m_sides[dirFace].m_label      = GetPrefixFromNum(ConvertLocationToType(pos, dirForward, pPS), (dirForward+1)%4, pPS) + "0" + MARKER + LEASTSIG;
    } else {
      s.m_sides[dirFace].m_strength   = 1;
      s.m_sides[dirFace].m_label      = GetPrefixFromNum(ConvertLocationToType(pos, dirForward, pPS), (dirForward+1)%4, pPS) + strMostsigdirFace + "0";
    }
    s.m_sides[dirReverse].m_label     = itos(iBase-1) + LEASTSIG;
    s.m_sides[dirBack].m_label        = std::string(SPACERMARKER) + BraceNumber(i, dirForward, pPS) + GetPrefixFromNum(i, dirForward, pPS);
    ts.push_back(s);

    //  The tile passing the SPACERMARKER from below to the right
    s.UsePrefix(true);
    s.m_color                         = SPACERCOLOR;
    s.m_display                       = std::string(SPACERMARKER);
    s.m_sides[dirForward].m_strength  = 1;
    s.m_sides[dirFace].m_strength     = 1;
    s.m_sides[dirReverse].m_strength  = 2;
    s.m_sides[dirBack].m_strength     = 1;
    s.m_sides[dirForward].m_label     = std::string("0");
    s.m_sides[dirFace].m_label        = std::string(SPACERMARKER) + BraceNumber(i, dirForward, pPS);
    s.m_sides[dirReverse].m_label     = std::string(SPACERMARKER) + BraceNumber(i, dirForward, pPS);
    s.m_sides[dirBack].m_label        = std::string(SPACERCOPYLEFT);
    ts.push_back(s);

    //  The tile passing the SPACERMARKER from the left upwards
    s.m_color                         = SPACERCOLOR;
    s.m_display                       = std::string(SPACERMARKER);
    s.m_sides[dirForward].m_strength  = 2;
    s.m_sides[dirFace].m_strength     = 1;
    s.m_sides[dirReverse].m_strength  = 1;
    s.m_sides[dirBack].m_strength     = 1;
    s.m_sides[dirForward].m_label     = std::string(SPACERMARKER) + BraceNumber(i, dirForward, pPS);
    s.m_sides[dirFace].m_label        = std::string(SPACERCOPYRIGHT) + BraceNumber(i, dirForward, pPS);
    s.m_sides[dirReverse].m_label     = std::string("0");
    s.m_sides[dirBack].m_label        = std::string(SPACERMARKER) + BraceNumber(i, dirForward, pPS);
    ts.push_back(s);

    //  The rightmost tile of the first spacer row (as long as there were > 2 bits flipped)
    s.m_color                         = SPACERCOLOR;
    s.m_display                       = itos(iBase-1) + LEASTSIG;
    s.m_sides[dirForward].m_strength  = 1;
    s.m_sides[dirReverse].m_strength  = 1;
    s.m_sides[dirBack].m_strength     = 1;
    s.m_sides[dirForward].m_label     = std::string("0") + LEASTSIG;
    s.m_sides[dirFace].m_bAddPrefix   = false;
    pos = i;
    if ((dirForward == WEST) || (dirForward == SOUTH)) {
      pos = GetReversedPosition(i, iBase);
      s.m_sides[dirFace].m_strength   = 2;
      s.m_sides[dirFace].m_label      = GetPrefixFromNum(ConvertLocationToType(pos, dirForward, pPS), (dirForward+1)%4, pPS) + "0" + MARKER + LEASTSIG;
    } else {
      s.m_sides[dirFace].m_strength   = 1;
      s.m_sides[dirFace].m_label      = GetPrefixFromNum(ConvertLocationToType(pos, dirForward, pPS), (dirForward+1)%4, pPS) + strMostsigdirFace + "0";
    }
    s.m_sides[dirReverse].m_label     = itos(iBase-1) + LEASTSIG;
    s.m_sides[dirBack].m_label        = std::string(MOSTSIG) + std::string(SPACERCOPYRIGHT) + BraceNumber(i, dirForward, pPS);
    ts.push_back(s);
  }

  for (int i=0; i<iBase; i++) {
  //  Spacer tiles in the top row and above a non-flipped bit
    Tile s;
    s.UsePrefix(true);
    s.m_color                         = SPACERCOLOR;
    s.m_display                       = itos(i);
    s.m_sides[dirForward].m_strength  = 1;
    s.m_sides[dirFace].m_strength     = 1;
    s.m_sides[dirReverse].m_strength  = 1;
    s.m_sides[dirBack].m_strength     = 1;
    s.m_sides[dirForward].m_label     = itos(i);
    s.m_sides[dirFace].m_label        = std::string(SPACERCOPYLEFT) + LEASTSIG;
    s.m_sides[dirReverse].m_label     = itos(i);
    s.m_sides[dirBack].m_label        = std::string(SPACERCOPYLEFT) + LEASTSIG;
    ts.push_back(s);

    //  Top right spacer tile in a block of at ldirFace two spacer rows
    s.UsePrefix(true);
    s.m_color                         = SPACERCOLOR;
    s.m_display                       = std::string("0") ;
    s.m_sides[dirForward].m_strength  = 2;
    s.m_sides[dirReverse].m_strength  = 2;
    s.m_sides[dirBack].m_strength     = 1;
    s.m_sides[dirForward].m_label     = std::string("0") + MARKER + std::string(LEASTSIG);
    s.m_sides[dirFace].m_bAddPrefix   = false;
    int pos = i;
    if ((dirForward == WEST) || (dirForward == SOUTH)) {
      pos = GetReversedPosition(i, iBase);
      s.m_sides[dirFace].m_strength   = 1;
      s.m_sides[dirFace].m_label      = GetPrefixFromNum(ConvertLocationToType(pos, dirForward, pPS), (dirForward+1)%4, pPS) + strMostsigdirFace + "0";
    } else {
      s.m_sides[dirFace].m_strength   = 2;
      s.m_sides[dirFace].m_label      = GetPrefixFromNum(ConvertLocationToType(pos, dirForward, pPS), (dirForward+1)%4, pPS) + "0" + MARKER + LEASTSIG;
    }
    s.m_sides[dirReverse].m_label     = std::string(SPACERMARKER) + BraceNumber(i, dirForward, pPS) + "0" + LEASTSIG;
    s.m_sides[dirBack].m_label        = std::string(SPACERCOPYLEFT) + LEASTSIG;
    ts.push_back(s);

    //  Tile immediately below the top right spacer tile in a block of two spacer rows (therefore the bottom right tile)
    s.UsePrefix(true);
    s.m_color                         = SPACERCOLOR;
    s.m_display                       = std::string("0") ;
    s.m_sides[dirForward].m_strength  = 2;
    s.m_sides[dirReverse].m_strength  = 1;
    s.m_sides[dirBack].m_strength     = 1;
    s.m_sides[dirForward].m_label     = std::string(SPACERMARKER) + BraceNumber(i, dirForward, pPS) + "0" + LEASTSIG;
    s.m_sides[dirFace].m_bAddPrefix   = false;
    pos = i;
    if ((dirForward == WEST) || (dirForward == SOUTH)) {
      pos = GetReversedPosition(i, iBase);
      s.m_sides[dirFace].m_strength   = 2;
      s.m_sides[dirFace].m_label      = GetPrefixFromNum(ConvertLocationToType(pos, dirForward, pPS), (dirForward+1)%4, pPS) + "0" + MARKER + LEASTSIG;
    } else {
      s.m_sides[dirFace].m_strength   = 1;
      s.m_sides[dirFace].m_label      = GetPrefixFromNum(ConvertLocationToType(pos, dirForward, pPS), (dirForward+1)%4, pPS) + strMostsigdirFace + "0";
    }
    s.m_sides[dirReverse].m_label     = itos(iBase-1) + LEASTSIG;
    s.m_sides[dirBack].m_label        = std::string(SPACERMARKER) + BraceNumber(i, dirForward, pPS);
    ts.push_back(s);

    //  Tile immediately below the top right spacer tile in a block of greater than two spacer rows
    s.UsePrefix(true);
    s.m_color                         = SPACERCOLOR;
    s.m_display                       = std::string("0") ;
    s.m_sides[dirForward].m_strength  = 2;
    s.m_sides[dirFace].m_strength     = 1;
    s.m_sides[dirReverse].m_strength  = 1;
    s.m_sides[dirBack].m_strength     = 1;
    s.m_sides[dirForward].m_label     = std::string(SPACERMARKER) + BraceNumber(i, dirForward, pPS) + "0" + LEASTSIG;
    s.m_sides[dirFace].m_bAddPrefix   = false;
    pos = i;
    if ((dirForward == WEST) || (dirForward == SOUTH)) {
      pos = GetReversedPosition(i, iBase);
    }
    s.m_sides[dirFace].m_label        = GetPrefixFromNum(ConvertLocationToType(pos, dirForward, pPS), (dirForward+1)%4, pPS) + "0";
    s.m_sides[dirReverse].m_label     = itos(0) + LEASTSIG;
    s.m_sides[dirBack].m_label        = std::string(SPACERMARKER) + BraceNumber(i, dirForward, pPS);
    ts.push_back(s);
  }

  //////////////////////////////////////////////////////////////////////
  //  Set type as prefix
  ts.SetPrefix(GetPrefixFromNum(iType, dirForward, pPS));

  return ts;
}


//////////////////////////////////////////////////////////////////////////////
//  Generate spacer block that begins this type of counter on its input
//  side (and initiates new types of counters on west or south if necessary)
TileSet CreateTransitionSpacerTileSet(int iBase, int iType, PointSet* pPS)
{
  TileSet ts;
  Tile t;

  int dirForward  = -1;
  int dirReverse  = -1;   //  This is the input direction
  int dirFace     = -1;
  int dirBack     = -1;

  Point* ptThis  = &pPS->at(iType);
  Point* ptInput = ptThis->GetInput();
  int numInput = ptInput->GetNum();
  dirReverse = ptThis->GetInputDir();
  if (ptInput == ptThis) {        //  Special case to handle case with iBase == 2 (which is only the Sierpinski Triangle)
    if (ptThis->GetX() == 0) {
      dirReverse = SOUTH;
    } else if (ptThis->GetY() == 0) {
      dirReverse = WEST;
    }
  }

  if ((dirReverse == NORTH) || (dirReverse == EAST)) {
    return CreateReverseTransitionTileSet(iBase, iType, pPS);
  }

  std::string strFacePrefix;

  switch(dirReverse) {
    case SOUTH:
      dirForward      = NORTH;
      dirFace         = EAST;
      dirBack         = WEST;
      strFacePrefix   = GetPrefixFromNum(iType, dirFace, pPS);
      break;
    case WEST:
      dirForward      = EAST;
      dirFace         = NORTH;
      dirBack         = SOUTH;
      strFacePrefix   = GetPrefixFromNum(iType, dirFace, pPS);
      break;
    default:
      break;
  }

  std::string strForwardPrefix  = GetPrefixFromNum(iType, dirForward, pPS);
  std::string strReversePrefix  = GetPrefixFromNum(numInput, dirReverse, pPS);

  bool bGrowsBack = false;
  std::string strBackPrefix     = "";
  if (ptThis->GetOutput(dirBack)) {
    bGrowsBack = true;
    strBackPrefix = GetPrefixFromNum(ptThis->GetOutput(dirBack)->GetNum(), dirBack, pPS);
  }

  std::cout << "Creating transition block for type " + itos(iType) + " counter (" + itos(ptThis->GetX()) + ", " + itos(ptThis->GetY()) + ").\n";
  std::cout << "\tInput is type " + itos(ptInput->GetNum()) + "(" + itos(ptInput->GetX()) + ", " + itos(ptInput->GetY()) + ") from the " + DirectionStringFromInt(dirReverse).c_str() + " with prefix " + strReversePrefix.c_str() + "\n";

  if (((iType != pPS->GetPointNumForLocation(Point(0, 1))) &&
      (iType != pPS->GetPointNumForLocation(Point(1, 0))))     ||
      (iBase == 2))
  {
    t.UsePrefix(false);
    t.m_display                       = std::string(MOSTSIG) + "0";

    t.m_sides[dirForward].m_strength  = 2;
    t.m_sides[dirForward].m_label     = strForwardPrefix + MOSTSIG + "0" + LEASTSIG;

    t.m_sides[dirFace].m_strength     = 2;
    t.m_sides[dirFace].m_label        = strFacePrefix + MOSTSIGNOGROW + "0" + LEASTSIG;

    t.m_sides[dirReverse].m_strength  = 2;
    t.m_sides[dirReverse].m_label     = strReversePrefix + MOSTSIG + itos(iBase-1) + LEASTSIG;

    t.m_sides[dirBack].m_bAddPrefix= false;
    if (!bGrowsBack) {
      t.m_sides[dirBack].m_strength   = 0;
      t.m_sides[dirBack].m_label      = NODISPLAY;
    } else {
      t.m_sides[dirBack].m_strength   = 2;
      t.m_sides[dirBack].m_label      = strBackPrefix + MOSTSIGNOGROW + itos(0) + LEASTSIG;
    }

    ts.push_back(t);

    //  Copy of the tile to initiate in non-growing MOSTSIG rows
    t.m_sides[dirForward].m_label    = strForwardPrefix + MOSTSIGNOGROW + "0" + LEASTSIG;
    t.m_sides[dirReverse].m_label    = strReversePrefix + MOSTSIGNOGROW + itos(iBase-1) + LEASTSIG;
    ts.push_back(t);
  }


  //  Special case tiles to connect to 'type 0' since they are only used for the very first iteration
  int num01 = pPS->GetPointNumForLocation(Point(0, 1));
  int num10 = pPS->GetPointNumForLocation(Point(1, 0));
  if ((iType == num01 && dirForward == NORTH) || (iType == num10 && dirForward == EAST)) {
    t.UsePrefix(true);

    t.m_sides[dirForward].m_bAddPrefix = false;
    t.m_sides[dirForward].m_strength = 2;
    t.m_sides[dirForward].m_label    = strForwardPrefix + MOSTSIG + itos(0) + LEASTSIG;

    t.m_sides[dirFace].m_bAddPrefix = false;
    t.m_sides[dirFace].m_strength  = 2;
    t.m_sides[dirFace].m_label     = strFacePrefix + MOSTSIGNOGROW + itos(0) + LEASTSIG;

    t.m_sides[dirReverse].m_bAddPrefix = false;
    t.m_sides[dirReverse].m_strength = 2;
    t.m_sides[dirReverse].m_label    = GetPrefixFromNum(0, dirForward, pPS) + MOSTSIG + itos(iBase-1) + LEASTSIG;

    t.m_sides[dirBack].m_strength  = 0;
    t.m_sides[dirBack].m_label     = NODISPLAY;

    ts.push_back(t);
  }

  //  Initial (bottom left) tile in this block
  if ((iType == pPS->GetPointNumForLocation(Point(0, 1))) ||
      (iType == pPS->GetPointNumForLocation(Point(1, 0)))) {         //  Check for special case where we need to grow another bit

    t.UsePrefix(true);
    t.m_display                 = std::string("0") + TRANSITIONMARKER;

    t.m_sides[dirForward].m_strength = 2;
    t.m_sides[dirForward].m_label    = std::string(TRANSITIONMARKER);  //  Not most significant position anymore!!

    t.m_sides[dirFace].m_strength  = 1;
    t.m_sides[dirFace].m_label     = std::string(TRANSITIONMARKER) + SPACERCOPYRIGHT;

    t.m_sides[dirReverse].m_bAddPrefix = false;
    t.m_sides[dirReverse].m_strength = 2;
    t.m_sides[dirReverse].m_label    = strReversePrefix + MOSTSIG + itos(iBase-1) + SPACERMARKER + MARKER;

    t.m_sides[dirBack].m_strength  = 2;
    t.m_sides[dirBack].m_label     = GROWFIBERMARKER;

    ts.push_back(t);


    t.UsePrefix(true);
    t.m_display                 = std::string("0") + TRANSITIONMARKER;

    t.m_sides[dirForward].m_strength = 2;
    t.m_sides[dirForward].m_label    = std::string(TRANSITIONMARKER) + "0" + LEASTSIG;

    t.m_sides[dirFace].m_bAddPrefix= false;
    t.m_sides[dirFace].m_strength  = 1;
    t.m_sides[dirFace].m_label     = strFacePrefix + MOSTSIGNOGROW + "0";

    t.m_sides[dirReverse].m_bAddPrefix = false;
    t.m_sides[dirReverse].m_strength = 2;
    //  Special case for Sierpinski Triangle due to small base 2 wraparound
    if (iBase == 2) {
      t.m_sides[dirReverse].m_label    = strReversePrefix + GROWFIBERMARKER + MOSTSIG + itos(iBase-1) + LEASTSIG;
    } else {
      t.m_sides[dirReverse].m_label    = strReversePrefix + MOSTSIG + itos(iBase-1) + LEASTSIG;
    }

    t.m_sides[dirBack].m_strength  = 2;
    t.m_sides[dirBack].m_label     = GROWFIBERMARKER;

    ts.push_back(t);


    t.UsePrefix(true);
    t.m_display                 = std::string(MOSTSIG) + "0" + TRANSITIONMARKER;

    t.m_sides[dirForward].m_strength = 1;
    t.m_sides[dirForward].m_label    = std::string(MOSTSIG) + "0";

    t.m_sides[dirFace].m_strength  = 2;
    t.m_sides[dirFace].m_label     = GROWFIBERMARKER;

    t.m_sides[dirReverse].m_strength = 0;
    t.m_sides[dirReverse].m_label    = NODISPLAY;

    t.m_sides[dirBack].m_bAddPrefix= false;
    if (!bGrowsBack) {
      t.m_sides[dirBack].m_strength  = 0;
      t.m_sides[dirBack].m_label     = NODISPLAY;
    } else {
      t.m_sides[dirBack].m_strength  = 1;
      t.m_sides[dirBack].m_label     = strBackPrefix + MOSTSIG + itos(0);
    }

    ts.push_back(t);

  } else {
    t.UsePrefix(true);
    t.m_display                 = std::string(MOSTSIG) + "0" + TRANSITIONMARKER;

    t.m_sides[dirForward].m_strength = 1;
    t.m_sides[dirForward].m_label    = std::string(MOSTSIG) + "0";

    t.m_sides[dirFace].m_strength  = 1;
    t.m_sides[dirFace].m_label     = TRANSITIONMARKER;

    t.m_sides[dirReverse].m_bAddPrefix= false;
    t.m_sides[dirReverse].m_strength = 2;
    t.m_sides[dirReverse].m_label    = strReversePrefix + std::string(MOSTSIG) + itos(iBase-1) + SPACERMARKER + MARKER;

    t.m_sides[dirBack].m_bAddPrefix= false;
    if (!bGrowsBack) {
      t.m_sides[dirBack].m_strength  = 0;
      t.m_sides[dirBack].m_label     = NODISPLAY;
    } else {
      t.m_sides[dirBack].m_strength  = 1;
      t.m_sides[dirBack].m_label     = strBackPrefix + MOSTSIGNOGROW + itos(0);
    }
    ts.push_back(t);

    t.m_sides[dirForward].m_label    = std::string(MOSTSIGNOGROW) + "0";
    t.m_sides[dirReverse].m_label    = strReversePrefix + std::string(MOSTSIGNOGROW) + itos(iBase-1) + SPACERMARKER + MARKER;
    ts.push_back(t);
  }

  //  Second to leftmost tiles of the bottom row of this block
  t.UsePrefix(true);
  t.m_display                 = std::string("0") + TRANSITIONMARKER;

  t.m_sides[dirForward].m_strength = 2;
  t.m_sides[dirForward].m_label    = TRANSITIONMARKER;

  t.m_sides[dirFace].m_strength  = 1;
  t.m_sides[dirFace].m_label     = std::string(TRANSITIONMARKER) + SPACERCOPYRIGHT;

  t.m_sides[dirReverse].m_bAddPrefix= false;
  t.m_sides[dirReverse].m_strength = 1;
  t.m_sides[dirReverse].m_label    = strReversePrefix + itos(iBase-1);

  t.m_sides[dirBack].m_strength  = 1;
  t.m_sides[dirBack].m_label     = TRANSITIONMARKER;

  ts.push_back(t);

  //  Interior tiles of the bottom row of this block
  t.UsePrefix(true);
  t.m_display                 = std::string("0");

  t.m_sides[dirForward].m_strength = 1;
  t.m_sides[dirForward].m_label    = "0";

  t.m_sides[dirFace].m_strength  = 1;
  t.m_sides[dirFace].m_label     = std::string(TRANSITIONMARKER) + SPACERCOPYRIGHT;

  t.m_sides[dirReverse].m_bAddPrefix= false;
  t.m_sides[dirReverse].m_strength = 1;
  t.m_sides[dirReverse].m_label    =strReversePrefix + itos(iBase-1);

  t.m_sides[dirBack].m_strength  = 1;
  t.m_sides[dirBack].m_label     = std::string(TRANSITIONMARKER) + SPACERCOPYRIGHT;

  ts.push_back(t);

  //  Rightmost tiles of the bottom row of this block (if width == 2)
  t.UsePrefix(true);
  t.m_display                 = std::string(TRANSITIONMARKER) + "0" + LEASTSIG;

  t.m_sides[dirForward].m_strength = 2;
  t.m_sides[dirForward].m_label    = std::string(TRANSITIONMARKER) + "0" + LEASTSIG;

  t.m_sides[dirFace].m_bAddPrefix= false;
  t.m_sides[dirFace].m_strength  = 1;
  t.m_sides[dirFace].m_label     = strFacePrefix + MOSTSIGNOGROW + "0";

  t.m_sides[dirReverse].m_bAddPrefix= false;
  t.m_sides[dirReverse].m_strength = 1;
  t.m_sides[dirReverse].m_label    = strReversePrefix + itos(iBase-1) + LEASTSIG;

  t.m_sides[dirBack].m_strength  = 1;
  t.m_sides[dirBack].m_label     = TRANSITIONMARKER;

  ts.push_back(t);

  //  Rightmost tiles of the bottom row of this block (if width > 2)
  t.UsePrefix(true);
  t.m_display                 = std::string("0") + LEASTSIG;

  t.m_sides[dirForward].m_strength = 1;
  t.m_sides[dirForward].m_label    = std::string("0") + LEASTSIG;

  t.m_sides[dirFace].m_bAddPrefix= false;
  t.m_sides[dirFace].m_strength  = 1;
  t.m_sides[dirFace].m_label     = strFacePrefix + MOSTSIGNOGROW + "0";

  t.m_sides[dirReverse].m_bAddPrefix= false;
  t.m_sides[dirReverse].m_strength = 1;
  t.m_sides[dirReverse].m_label    = strReversePrefix + itos(iBase-1) + LEASTSIG;

  t.m_sides[dirBack].m_strength  = 1;
  t.m_sides[dirBack].m_label     = std::string(TRANSITIONMARKER) + SPACERCOPYRIGHT;

  ts.push_back(t);

  //  Leftmost tiles in this block but not bottom or top
  t.UsePrefix(true);
  t.m_display                   = std::string(MOSTSIG) + "0";

  t.m_sides[dirForward].m_strength   = 1;
  t.m_sides[dirForward].m_label      = std::string(MOSTSIG) + "0";

  t.m_sides[dirFace].m_strength    = 1;
  t.m_sides[dirFace].m_label       = std::string(SPACERCOPYLEFT) + TRANSITIONMARKER;

  t.m_sides[dirReverse].m_strength   = 1;
  t.m_sides[dirReverse].m_label      = std::string(MOSTSIG) + "0";

  t.m_sides[dirBack].m_bAddPrefix  = false;
  if (!bGrowsBack) {
    t.m_sides[dirBack].m_strength  = 0;
    t.m_sides[dirBack].m_label     = NODISPLAY;
  } else {
    t.m_sides[dirBack].m_strength  = 1;
    t.m_sides[dirBack].m_label     = strBackPrefix + itos(0);
  }
  ts.push_back(t);

  //  Copy for fixed-width instance
  t.m_sides[dirForward].m_label      = std::string(MOSTSIGNOGROW) + "0";
  t.m_sides[dirReverse].m_label      = std::string(MOSTSIGNOGROW) + "0";
  ts.push_back(t);

  //  Interior tiles left of the transition marker, but not in bottom or top rows
  t.UsePrefix(true);
  t.m_display                 = std::string("0");

  t.m_sides[dirForward].m_strength = 1;
  t.m_sides[dirForward].m_label    = "0";

  t.m_sides[dirFace].m_strength  = 1;
  t.m_sides[dirFace].m_label     = std::string(SPACERCOPYLEFT) + TRANSITIONMARKER;

  t.m_sides[dirReverse].m_strength = 1;
  t.m_sides[dirReverse].m_label    = "0";

  t.m_sides[dirBack].m_strength  = 1;
  t.m_sides[dirBack].m_label     = std::string(SPACERCOPYLEFT) + TRANSITIONMARKER;

  ts.push_back(t);

  //  Interior tiles right of the transition marker, but not in bottom or top rows
  t.UsePrefix(true);
  t.m_display                 = std::string("0");

  t.m_sides[dirForward].m_strength = 1;
  t.m_sides[dirForward].m_label    = "0";

  t.m_sides[dirFace].m_strength  = 1;
  t.m_sides[dirFace].m_label     = std::string(SPACERCOPYRIGHT) + TRANSITIONMARKER;

  t.m_sides[dirReverse].m_strength = 1;
  t.m_sides[dirReverse].m_label    = "0";

  t.m_sides[dirBack].m_strength  = 1;
  t.m_sides[dirBack].m_label     = std::string(SPACERCOPYRIGHT) + TRANSITIONMARKER;

  ts.push_back(t);

  //  Interior tiles passing the transition marker from below to the right, but not in bottom or top rows
  t.UsePrefix(true);
  t.m_display                 = std::string("0") + TRANSITIONMARKER;

  t.m_sides[dirForward].m_strength = 1;
  t.m_sides[dirForward].m_label    = "0";

  t.m_sides[dirFace].m_strength  = 1;
  t.m_sides[dirFace].m_label     = TRANSITIONMARKER;

  t.m_sides[dirReverse].m_strength = 2;
  t.m_sides[dirReverse].m_label    = TRANSITIONMARKER;

  t.m_sides[dirBack].m_strength  = 1;
  t.m_sides[dirBack].m_label     = std::string(SPACERCOPYLEFT) + TRANSITIONMARKER;

  ts.push_back(t);

  //  Interior tiles passing the transition marker from the right to above, but not in bottom or top rows
  t.UsePrefix(true);
  t.m_display                 = std::string("0") + TRANSITIONMARKER;

  t.m_sides[dirForward].m_strength = 2;
  t.m_sides[dirForward].m_label    = TRANSITIONMARKER;

  t.m_sides[dirFace].m_strength  = 1;
  t.m_sides[dirFace].m_label     = std::string(SPACERCOPYRIGHT) + TRANSITIONMARKER;

  t.m_sides[dirReverse].m_strength = 1;
  t.m_sides[dirReverse].m_label    = "0";

  t.m_sides[dirBack].m_strength  = 1;
  t.m_sides[dirBack].m_label     = TRANSITIONMARKER;

  ts.push_back(t);

  //  Rightmost tiles right of the transition marker, but not in bottom or top rows
  t.UsePrefix(true);
  t.m_display                 = std::string("0") + LEASTSIG;

  t.m_sides[dirForward].m_strength = 1;
  t.m_sides[dirForward].m_label    = std::string("0") + LEASTSIG;

  t.m_sides[dirFace].m_bAddPrefix= false;
  t.m_sides[dirFace].m_strength  = 1;
  t.m_sides[dirFace].m_label     = strFacePrefix + std::string("0");

  t.m_sides[dirReverse].m_strength = 1;
  t.m_sides[dirReverse].m_label    = std::string("0") + LEASTSIG;

  t.m_sides[dirBack].m_strength  = 1;
  t.m_sides[dirBack].m_label     = std::string(SPACERCOPYRIGHT) + TRANSITIONMARKER;

  ts.push_back(t);

  //  Rightmost tile that passes TRANSITIONMARKER from the left to above
  t.UsePrefix(true);
  t.m_display                 = std::string(TRANSITIONMARKER) + "0" + LEASTSIG;

  t.m_sides[dirForward].m_strength = 2;
  t.m_sides[dirForward].m_label    = std::string(TRANSITIONMARKER) + "0" + LEASTSIG;

  t.m_sides[dirFace].m_bAddPrefix= false;
  t.m_sides[dirFace].m_strength  = 1;
  t.m_sides[dirFace].m_label     = strFacePrefix + "0";

  t.m_sides[dirReverse].m_strength = 1;
  t.m_sides[dirReverse].m_label    = std::string("0") + LEASTSIG;

  t.m_sides[dirBack].m_strength  = 1;
  t.m_sides[dirBack].m_label     = std::string(TRANSITIONMARKER);

  ts.push_back(t);

  //  Top right tile of this block
  t.UsePrefix(true);
  t.m_display                 = std::string(MOSTSIG) + "0" + LEASTSIG;

  t.m_sides[dirForward].m_bAddPrefix = false;
  t.m_sides[dirForward].m_strength = 2;
  t.m_sides[dirForward].m_label    = strForwardPrefix + "0" + MARKER + LEASTSIG;

  t.m_sides[dirFace].m_bAddPrefix= false;
  t.m_sides[dirFace].m_strength  = 2;
  t.m_sides[dirFace].m_label     = strFacePrefix + std::string("0") + MARKER + LEASTSIG;

  t.m_sides[dirReverse].m_strength = 2;
  t.m_sides[dirReverse].m_label    = std::string(TRANSITIONMARKER) + "0" + LEASTSIG;

  t.m_sides[dirBack].m_strength  = 1;
  t.m_sides[dirBack].m_label     = std::string(MOSTSIG) + SPACERCOPYLEFT + TRANSITIONMARKER;

  ts.push_back(t);

  //  Interior tiles of the top row of this block
  t.UsePrefix(true);
  t.m_display                 = std::string(MOSTSIG) + "0";

  t.m_sides[dirForward].m_bAddPrefix = false;
  t.m_sides[dirForward].m_strength  = 1;
  t.m_sides[dirForward].m_label     = strForwardPrefix + "0";

  t.m_sides[dirFace].m_strength  = 1;
  t.m_sides[dirFace].m_label     = std::string(MOSTSIG) + SPACERCOPYLEFT + TRANSITIONMARKER;

  t.m_sides[dirReverse].m_strength = 1;
  t.m_sides[dirReverse].m_label    = "0";

  t.m_sides[dirBack].m_strength  = 1;
  t.m_sides[dirBack].m_label     = std::string(MOSTSIG) + SPACERCOPYLEFT + TRANSITIONMARKER;

  ts.push_back(t);

  //  Top left tile of this block
  t.UsePrefix(true);
  t.m_display                   = std::string(MOSTSIG) + "0";

  t.m_sides[dirForward].m_bAddPrefix = false;
  t.m_sides[dirForward].m_strength   = 1;
  t.m_sides[dirForward].m_label      = strForwardPrefix + MOSTSIG + "0";

  t.m_sides[dirFace].m_strength    = 1;
  t.m_sides[dirFace].m_label       = std::string(MOSTSIG) + SPACERCOPYLEFT + TRANSITIONMARKER;

  t.m_sides[dirReverse].m_strength   = 1;
  t.m_sides[dirReverse].m_label      = std::string(MOSTSIG) + "0";

  t.m_sides[dirBack].m_bAddPrefix  = false;
  if (!bGrowsBack) {
    t.m_sides[dirBack].m_strength  = 0;
    t.m_sides[dirBack].m_label     = NODISPLAY;
  } else {
    t.m_sides[dirBack].m_strength  = 2;
    t.m_sides[dirBack].m_label     = strBackPrefix + itos(0) + MARKER + LEASTSIG;
  }
  ts.push_back(t);

  //  Copy for fixed-width instance
  t.m_sides[dirForward].m_label      = strForwardPrefix + MOSTSIGNOGROW + "0";
  t.m_sides[dirReverse].m_label      = std::string(MOSTSIGNOGROW) + "0";
  ts.push_back(t);

  //////////////////////////////////////////////////////////////////////
  //  Set type as prefix
  ts.SetPrefix(GetPrefixFromNum(iType, -1, pPS));

  //  Set the color for these tiletypes
  for (size_t i=0; i<ts.size(); i++) {
    ts[i].m_color = TRANSITIONSPACERCOLOR;
  }

  return ts;
}


TileSet CreateReverseTransitionTileSet(int iBase, int iType, PointSet* pPS)
{
  TileSet ts;

  Point* ptThis  = &pPS->at(iType);
  Point* ptInput = ptThis->GetInput();
  int numInput = ptInput->GetNum();

  int dirTerm       = EAST;
  int dirInit       = NORTH;
  int dirOutAcross  = WEST;
  int dirOutBack    = SOUTH;
  if (ptThis->GetInputDir() == NORTH) {
    dirTerm         = NORTH;
    dirInit         = EAST;
    dirOutAcross    = SOUTH;
    dirOutBack      = WEST;
  }

  std::string strPrefixTerm = GetPrefixFromNum(iType, dirTerm, pPS);
  std::string strPrefixInit = GetPrefixFromNum(iType, dirInit, pPS);
  std::string strPrefixOutAcross;
  std::string strPrefixOutBack;

  bool bHasOutAcross    = false;
  if (ptThis->GetOutput(dirOutAcross)) {
    bHasOutAcross       = true;
    strPrefixOutAcross  = GetPrefixFromNum(ptThis->GetOutput(dirOutAcross)->GetNum(), dirOutAcross, pPS);
  }
  bool bHasOutBack      = false;
  if (ptThis->GetOutput(dirOutBack)) {
    bHasOutBack         = true;
    strPrefixOutBack    = GetPrefixFromNum(ptThis->GetOutput(dirOutBack)->GetNum(), dirOutBack, pPS);
  }

  std::cout << "Creating reverse growth transition block for type " + itos(iType) + " counter (" + itos(ptThis->GetX()) + ", " + itos(ptThis->GetY()) + ").\n\tInput is from " + DirectionStringFromInt(ptThis->GetDirectionToPoint(*ptInput)).c_str() + ".\n";

  //  Create the tile for single, non-increasing width transition block
  Tile t;
  t.UsePrefix(false);
  t.m_display                           = std::string(MOSTSIG) + "0";

  t.m_sides[dirTerm].m_strength         = 2;
  t.m_sides[dirTerm].m_label            = strPrefixTerm + MOSTSIGNOGROW + itos(iBase-1) + LEASTSIG;

  t.m_sides[dirInit].m_strength         = 2;
  t.m_sides[dirInit].m_label            = strPrefixInit + MOSTSIGNOGROW + "0" + LEASTSIG;

  if (!bHasOutAcross) {
    t.m_sides[dirOutAcross].m_strength  = 0;
    t.m_sides[dirOutAcross].m_label     = NODISPLAY;
  } else {
    t.m_sides[dirOutAcross].m_strength  = 2;
    t.m_sides[dirOutAcross].m_label     = strPrefixOutAcross + MOSTSIGNOGROW + "0" + LEASTSIG;
  }

  if (!bHasOutBack) {
    t.m_sides[dirOutBack].m_strength    = 0;
    t.m_sides[dirOutBack].m_label       = NODISPLAY;
  } else {
    t.m_sides[dirOutBack].m_strength    = 2;
    t.m_sides[dirOutBack].m_label       = strPrefixOutBack + MOSTSIGNOGROW + "0" + LEASTSIG;
  }

  ts.push_back(t);

  ////////////////////////////////////////////////////
  //  Tiles for blocks of width >= 2

  //  Initiating tile
  t.UsePrefix(true);
  t.m_display                           = std::string(MOSTSIGNOGROW) + "0" + TRANSITIONMARKER;

  t.m_sides[dirTerm].m_bAddPrefix       = false;
  t.m_sides[dirTerm].m_strength         = 2;
  t.m_sides[dirTerm].m_label            = strPrefixTerm + MOSTSIGNOGROW + itos(iBase-1) + SPACERMARKER + MARKER;

  t.m_sides[dirInit].m_strength         = 1;
  t.m_sides[dirInit].m_label            = TRANSITIONMARKER;

  t.m_sides[dirOutAcross].m_strength    = 1;
  t.m_sides[dirOutAcross].m_label       = std::string(MOSTSIGNOGROW) + "0";

  t.m_sides[dirOutBack].m_bAddPrefix    = false;
  if (!bHasOutBack) {
    t.m_sides[dirOutBack].m_strength    = 0;
    t.m_sides[dirOutBack].m_label       = NODISPLAY;
  } else {
    t.m_sides[dirOutBack].m_strength    = 2;
    t.m_sides[dirOutBack].m_label       = strPrefixOutBack + "0" + MARKER + LEASTSIG;
  }

  ts.push_back(t);

  //  Interior tile of initiating row that receives marker from the left
  t.UsePrefix(true);
  t.m_display                           = std::string("0") + TRANSITIONMARKER;

  t.m_sides[dirTerm].m_bAddPrefix       = false;
  t.m_sides[dirTerm].m_strength         = 1;
  t.m_sides[dirTerm].m_label            = strPrefixTerm + itos(iBase-1);

  t.m_sides[dirInit].m_strength         = 1;
  t.m_sides[dirInit].m_label            = std::string(TRANSITIONMARKER) + SPACERCOPYRIGHT;

  t.m_sides[dirOutAcross].m_strength    = 2;
  t.m_sides[dirOutAcross].m_label       = TRANSITIONMARKER;

  t.m_sides[dirOutBack].m_strength      = 1;
  t.m_sides[dirOutBack].m_label         = TRANSITIONMARKER;

  ts.push_back(t);

  //  Interior tile of initiating row that doesn't receive the marker
  t.UsePrefix(true);
  t.m_display                           = std::string("0");

  t.m_sides[dirTerm].m_bAddPrefix       = false;
  t.m_sides[dirTerm].m_strength         = 1;
  t.m_sides[dirTerm].m_label            = strPrefixTerm + itos(iBase-1);

  t.m_sides[dirInit].m_strength         = 1;
  t.m_sides[dirInit].m_label            = std::string(TRANSITIONMARKER) + SPACERCOPYRIGHT;

  t.m_sides[dirOutAcross].m_strength    = 1;
  t.m_sides[dirOutAcross].m_label       = "0";

  t.m_sides[dirOutBack].m_strength      = 1;
  t.m_sides[dirOutBack].m_label         = std::string(TRANSITIONMARKER) + SPACERCOPYRIGHT;

  ts.push_back(t);

  //  Last tile of initiating row if width == 2 (so it receives the marker)
  t.UsePrefix(true);
  t.m_display                           = std::string("0") + LEASTSIG;

  t.m_sides[dirTerm].m_bAddPrefix       = false;
  t.m_sides[dirTerm].m_strength         = 1;
  t.m_sides[dirTerm].m_label            = strPrefixTerm + itos(iBase-1) + LEASTSIG;

  t.m_sides[dirInit].m_bAddPrefix       = false;
  t.m_sides[dirInit].m_strength         = 2;
  t.m_sides[dirInit].m_label            = strPrefixInit + "0" + MARKER + LEASTSIG;

  t.m_sides[dirOutAcross].m_strength    = 2;
  t.m_sides[dirOutAcross].m_label       = std::string(TRANSITIONMARKER) + LEASTSIG;

  t.m_sides[dirOutBack].m_strength      = 1;
  t.m_sides[dirOutBack].m_label         = TRANSITIONMARKER;

  ts.push_back(t);

  //  Last tile of initiating row if width > 2 (so it doesn't receive the marker)
  t.UsePrefix(true);
  t.m_display                           = std::string("0") + LEASTSIG;

  t.m_sides[dirTerm].m_bAddPrefix       = false;
  t.m_sides[dirTerm].m_strength         = 1;
  t.m_sides[dirTerm].m_label            = strPrefixTerm + itos(iBase-1) + LEASTSIG;

  t.m_sides[dirInit].m_bAddPrefix       = false;
  t.m_sides[dirInit].m_strength         = 2;
  t.m_sides[dirInit].m_label            = strPrefixInit + "0" + MARKER + LEASTSIG;

  t.m_sides[dirOutAcross].m_strength    = 1;
  t.m_sides[dirOutAcross].m_label       = std::string("0") + LEASTSIG;

  t.m_sides[dirOutBack].m_strength      = 1;
  t.m_sides[dirOutBack].m_label         = std::string(TRANSITIONMARKER) + SPACERCOPYRIGHT;

  ts.push_back(t);

  //  Mostsig tile of interior rows
  t.UsePrefix(true);
  t.m_display                           = std::string(MOSTSIGNOGROW) + "0";

  t.m_sides[dirTerm].m_strength         = 1;
  t.m_sides[dirTerm].m_label            = std::string(MOSTSIGNOGROW) + "0";

  t.m_sides[dirInit].m_strength         = 1;
  t.m_sides[dirInit].m_label            = std::string(SPACERCOPYLEFT) + TRANSITIONMARKER;

  t.m_sides[dirOutAcross].m_strength    = 1;
  t.m_sides[dirOutAcross].m_label       = std::string(MOSTSIGNOGROW) + "0";

  t.m_sides[dirOutBack].m_bAddPrefix    = false;
  if (!bHasOutBack) {
    t.m_sides[dirOutBack].m_strength    = 0;
    t.m_sides[dirOutBack].m_label       = NODISPLAY;
  } else {
    t.m_sides[dirOutBack].m_strength    = 1;
    t.m_sides[dirOutBack].m_label       = strPrefixOutBack + "0";
  }

  ts.push_back(t);

  //  Interior tile of interior rows left of marker
  t.UsePrefix(true);
  t.m_display                           = "0";

  t.m_sides[dirTerm].m_strength         = 1;
  t.m_sides[dirTerm].m_label            = "0";

  t.m_sides[dirInit].m_strength         = 1;
  t.m_sides[dirInit].m_label            = std::string(SPACERCOPYLEFT) + TRANSITIONMARKER;

  t.m_sides[dirOutAcross].m_strength    = 1;
  t.m_sides[dirOutAcross].m_label       = "0";

  t.m_sides[dirOutBack].m_strength      = 1;
  t.m_sides[dirOutBack].m_label         = std::string(SPACERCOPYLEFT) + TRANSITIONMARKER;

  ts.push_back(t);

  //  Interior tile of interior rows right of marker
  t.UsePrefix(true);
  t.m_display                           = "0";

  t.m_sides[dirTerm].m_strength         = 1;
  t.m_sides[dirTerm].m_label            = "0";

  t.m_sides[dirInit].m_strength         = 1;
  t.m_sides[dirInit].m_label            = std::string(TRANSITIONMARKER) + SPACERCOPYRIGHT;

  t.m_sides[dirOutAcross].m_strength    = 1;
  t.m_sides[dirOutAcross].m_label       = "0";

  t.m_sides[dirOutBack].m_strength      = 1;
  t.m_sides[dirOutBack].m_label         = std::string(TRANSITIONMARKER) + SPACERCOPYRIGHT;

  ts.push_back(t);

  //  Interior tile of interior rows that take marker from left and pass it forward
  t.UsePrefix(true);
  t.m_display                           = TRANSITIONMARKER;

  t.m_sides[dirTerm].m_strength         = 1;
  t.m_sides[dirTerm].m_label            = "0";

  t.m_sides[dirInit].m_strength         = 1;
  t.m_sides[dirInit].m_label            = std::string(TRANSITIONMARKER) + SPACERCOPYRIGHT;

  t.m_sides[dirOutAcross].m_strength    = 2;
  t.m_sides[dirOutAcross].m_label       = TRANSITIONMARKER;

  t.m_sides[dirOutBack].m_strength      = 1;
  t.m_sides[dirOutBack].m_label         = TRANSITIONMARKER;

  ts.push_back(t);

  //  Interior tile of interior rows that take marker from behind and pass it to the right
  t.UsePrefix(true);
  t.m_display                           = TRANSITIONMARKER;

  t.m_sides[dirTerm].m_strength         = 2;
  t.m_sides[dirTerm].m_label            = TRANSITIONMARKER;

  t.m_sides[dirInit].m_strength         = 1;
  t.m_sides[dirInit].m_label            = TRANSITIONMARKER;

  t.m_sides[dirOutAcross].m_strength    = 1;
  t.m_sides[dirOutAcross].m_label       = "0";

  t.m_sides[dirOutBack].m_strength      = 1;
  t.m_sides[dirOutBack].m_label         = std::string(SPACERCOPYLEFT) + TRANSITIONMARKER;

  ts.push_back(t);

  //  Leastsig tile of interior rows
  t.UsePrefix(true);
  t.m_display                           = std::string("0") + LEASTSIG;

  t.m_sides[dirTerm].m_strength         = 1;
  t.m_sides[dirTerm].m_label            = std::string("0") + LEASTSIG;

  t.m_sides[dirInit].m_bAddPrefix       = false;
  t.m_sides[dirInit].m_strength         = 1;
  t.m_sides[dirInit].m_label            = strPrefixInit + "0";

  t.m_sides[dirOutAcross].m_strength    = 1;
  t.m_sides[dirOutAcross].m_label       = std::string("0") + LEASTSIG;

  t.m_sides[dirOutBack].m_strength      = 1;
  t.m_sides[dirOutBack].m_label         = std::string(TRANSITIONMARKER) + SPACERCOPYRIGHT;

  ts.push_back(t);

  //  Leastsig tile of interior row (2nd to last row) that receives marker from the left and passes forward
  t.UsePrefix(true);
  t.m_display                           = std::string(TRANSITIONMARKER) + "0" + LEASTSIG;

  t.m_sides[dirTerm].m_strength         = 1;
  t.m_sides[dirTerm].m_label            = std::string("0") + LEASTSIG;

  t.m_sides[dirInit].m_bAddPrefix       = false;
  t.m_sides[dirInit].m_strength         = 1;
  t.m_sides[dirInit].m_label            = strPrefixInit + "0";

  t.m_sides[dirOutAcross].m_strength    = 2;
  t.m_sides[dirOutAcross].m_label       = std::string(TRANSITIONMARKER) + LEASTSIG;

  t.m_sides[dirOutBack].m_strength      = 1;
  t.m_sides[dirOutBack].m_label         = TRANSITIONMARKER;

  ts.push_back(t);

  //  Mostsig tile of last row
  t.UsePrefix(true);
  t.m_display                           = std::string(MOSTSIGNOGROW) + "0";

  t.m_sides[dirTerm].m_strength         = 1;
  t.m_sides[dirTerm].m_label            = std::string(MOSTSIGNOGROW) + "0";

  t.m_sides[dirInit].m_strength         = 1;
  t.m_sides[dirInit].m_label            = std::string(MOSTSIGNOGROW) + SPACERCOPYLEFT + TRANSITIONMARKER;

  t.m_sides[dirOutAcross].m_bAddPrefix  = false;
  if (!bHasOutAcross) {
    t.m_sides[dirOutAcross].m_strength  = 0;
    t.m_sides[dirOutAcross].m_label     = NODISPLAY;
  } else {
    t.m_sides[dirOutAcross].m_strength  = 1;
    t.m_sides[dirOutAcross].m_label     = strPrefixOutAcross + MOSTSIGNOGROW + "0";
  }

  t.m_sides[dirOutBack].m_bAddPrefix    = false;
  if (!bHasOutBack) {
    t.m_sides[dirOutBack].m_strength    = 0;
    t.m_sides[dirOutBack].m_label       = NODISPLAY;
  } else {
    t.m_sides[dirOutBack].m_strength    = 1;
    t.m_sides[dirOutBack].m_label       = strPrefixOutBack + MOSTSIGNOGROW + std::string("0");
  }

  ts.push_back(t);

  //  Interior tile of last row (left of marker)
  t.UsePrefix(true);
  t.m_display                           = "0";

  t.m_sides[dirTerm].m_strength         = 1;
  t.m_sides[dirTerm].m_label            = "0";

  t.m_sides[dirInit].m_strength         = 1;
  t.m_sides[dirInit].m_label            = std::string(MOSTSIGNOGROW) + SPACERCOPYLEFT + TRANSITIONMARKER;

  t.m_sides[dirOutAcross].m_bAddPrefix  = false;
  if (!bHasOutAcross) {
    t.m_sides[dirOutAcross].m_strength  = 0;
    t.m_sides[dirOutAcross].m_label     = NODISPLAY;
  } else {
    t.m_sides[dirOutAcross].m_strength  = 1;
    t.m_sides[dirOutAcross].m_label     = strPrefixOutAcross + "0";
  }

  t.m_sides[dirOutBack].m_strength      = 1;
  t.m_sides[dirOutBack].m_label         = std::string(MOSTSIGNOGROW) + SPACERCOPYLEFT + TRANSITIONMARKER;

  ts.push_back(t);

  //  First tile of last row (with marker)
  t.UsePrefix(true);
  t.m_display                           = std::string(TRANSITIONMARKER) + "0" + LEASTSIG;

  t.m_sides[dirTerm].m_strength         = 2;
  t.m_sides[dirTerm].m_label            = std::string(TRANSITIONMARKER) + LEASTSIG;

  t.m_sides[dirInit].m_bAddPrefix       = false;
  t.m_sides[dirInit].m_strength         = 1;
  t.m_sides[dirInit].m_label            = strPrefixInit + std::string(MOSTSIGNOGROW) + "0";

  t.m_sides[dirOutAcross].m_bAddPrefix  = false;
  if (!bHasOutAcross) {
    t.m_sides[dirOutAcross].m_strength  = 0;
    t.m_sides[dirOutAcross].m_label     = NODISPLAY;
  } else {
    t.m_sides[dirOutAcross].m_strength  = 2;
    t.m_sides[dirOutAcross].m_label     = strPrefixOutAcross + "0" + MARKER + LEASTSIG;
  }

  t.m_sides[dirOutBack].m_strength      = 1;
  t.m_sides[dirOutBack].m_label         = std::string(MOSTSIGNOGROW) + SPACERCOPYLEFT + TRANSITIONMARKER;

  ts.push_back(t);


  //////////////////////////////////////////////////////////////////////
  //  Set type as prefix
  ts.SetPrefix(GetPrefixFromNum(iType, -1, pPS));

  //  Set the color for these tiletypes
  for (size_t i=0; i<ts.size(); i++) {
    ts[i].m_color = TRANSITIONSPACERCOLOR;
  }

  return ts;
}

bool GenerateFiberedSSFTileset(PointSet* pPSIn, TileSet* pTS, int quadrant)
{
  TileSet  ts;
  PointSet ps;

  int xMin = 0;
  int yMin = 0;
  int xMax = 0;
  int yMax = 0;
  for (size_t i=0; i<pPSIn->size(); i++) {
    if (pPSIn->at(i).GetX() < xMin) xMin = pPSIn->at(i).GetX();
    if (pPSIn->at(i).GetY() < yMin) yMin = pPSIn->at(i).GetY();
    if (pPSIn->at(i).GetX() > xMax) xMax = pPSIn->at(i).GetX();
    if (pPSIn->at(i).GetY() > yMax) yMax = pPSIn->at(i).GetY();
  }
  for (size_t i=0; i<pPSIn->size(); i++) {
    Point p(pPSIn->at(i));
    p.SetX(p.GetX() - xMin);
    p.SetY(p.GetY() - yMin);
    ps.push_back(p);
  }
  ps.SetOrigin(ps.GetPointNumForLocation(Point(0, 0)));

  std::cout << "PointSet: ";
  for (size_t i=0; i<ps.size(); i++)  {
    std::cout << "(" << ps.at(i).GetX() << "," << ps.at(i).GetY() << ")";
  }
  std::cout << "\n\n";

  //  First, transform pointSet back to first quadrant if necessary, then rotate after creation
  for (size_t i=0; i<ps.size(); i++) {
    if (quadrant == QUAD2) {
      ps.at(i).SetY(abs(ps.at(i).GetY()));
      yMax = abs(yMin);
    } else if (quadrant == QUAD3) {
      ps.at(i).SetX(abs(ps.at(i).GetX()));
      ps.at(i).SetY(abs(ps.at(i).GetY()));
      xMax = abs(xMin);
      yMax = abs(yMin);
    } else if (quadrant == QUAD4) {
      ps.at(i).SetX(abs(ps.at(i).GetX()));
      xMax = abs(xMin);
    }
  }

  if (xMax != yMax) {
    std::cout << "\n\nERROR:  Illegal fractal definition:  The input points do not form a square.\n\n";
    return false;
  }
  int c = (xMax > yMax ? xMax : yMax);
  std::cout << "c = " + itos(c) + "(base = " + itos(c+1) + ")\n";

  if (!ps.CalculateInputs(true)) {
    std::cout << "\nERROR:  Illegal fractal definition:  The points aren't connected.  Aborting.\n\n";
    return false;
  }

  for (int i=0; i<=c; i++) {
    Point h(i,0);
    Point v(0,i);
    if ((ps.GetPointNumForLocation(h) < 0) || (ps.GetPointNumForLocation(v) < 0)) {
      std::cout << "\n\nERROR:  Illegal fractal definition:  All points along the x and y axes must be included.\n\n";
      return false;
    }
  }
  std::cout << "Valid fractal definition.\n";

  ps.CalculateOutputs();

  //  Set inputs/outputs to 'wrap around'
  Point ptY(0, c);
  Point* pPt = &ps.at(ps.GetPointNumForLocation(ptY));
  pPt->SetOutput(NORTH, &ps.at(ps.GetPointNumForLocation(Point(0, 1))));
  Point* pPt2 = &ps.at(ps.GetPointNumForLocation(Point(0, 1)));
  pPt2->SetInput(pPt);
  pPt2->SetInputDir(SOUTH);

  Point ptX(c, 0);
  pPt = &ps.at(ps.GetPointNumForLocation(ptX));
  pPt->SetOutput(EAST, &ps.at(ps.GetPointNumForLocation(Point(1, 0))));
  pPt2 = &ps.at(ps.GetPointNumForLocation(Point(1, 0)));
  pPt2->SetInput(pPt);
  pPt2->SetInputDir(WEST);

  std::cout << "Outputs:\n";
  for (size_t i=0; i<ps.size(); i++) {
    Point* ptN = ps.at(i).GetOutput(NORTH);
    Point* ptE = ps.at(i).GetOutput(EAST);
    Point* ptS = ps.at(i).GetOutput(SOUTH);
    Point* ptW = ps.at(i).GetOutput(WEST);

    std::cout << std::string("Type ") + itos(i) + "[" + itos(ps.at(i).GetX()) + "," + itos(ps.at(i).GetY()) + "] :";
    if (ptN) {
      std::cout << std::string("N ") + itos(ptN->GetNum()) + "[" + itos(ptN->GetX()) + "," + itos(ptN->GetY()) + "],";
    }
    if (ptE) {
      std::cout << std::string("E ") + itos(ptE->GetNum()) + "[" + itos(ptE->GetX()) + "," + itos(ptE->GetY()) + "],";
    }
    if (ptS) {
      std::cout << std::string("S ") + itos(ptS->GetNum()) + "[" + itos(ptS->GetX()) + "," + itos(ptS->GetY()) + "],";
    }
    if (ptW) {
      std::cout << std::string("W ") + itos(ptW->GetNum()) + "[" + itos(ptW->GetX()) + "," + itos(ptW->GetY()) + "]";
    }
    std::cout << "\n";
  }

  std::cout << "Generating a tileset for a fibered version of the fractal...\n";

  //  Create tiles for F1 that are not on either axis
  TileSet tsF1;
  for (size_t i=0; i<ps.size(); i++) {
    std::string disp = std::string("(") + ps.at(i).ToString() + ")";
    Tile f(disp.c_str());
    f.m_display = disp;

    bool bOnAxis = false;
    if ((ps.at(i).GetX() == 0) || (ps.at(i).GetY() == 0)) {
      f.UsePrefix(true);
      bOnAxis = true;
    } else {
      f.UsePrefix(false);
    }

    //  Create input side
    Point* ptIn = ps.at(i).GetInput();
    if (ptIn) {
      int input = ps.at(i).GetDirectionToPoint(*ptIn);
      if (input >= 0) {
        f.m_sides[input].m_strength = 2;
        f.m_sides[input].m_label    = ps.at(i).GetOffset(input, 1).ToString();
      }
    }

    //  Create output sides
    for (size_t j=0; j<ps.size(); j++) {
      Point* ptIn2 = ps.at(j).GetInput();
      if (ptIn2) {
        int input2 = ps.at(j).GetDirectionToPoint(*ptIn2);
        if (ps.at(i) == ps.at(j).GetOffset(input2, 1)) {
          if (bOnAxis && (ps.at(j).GetX() != 0) && (ps.at(j).GetY() != 0)) {
            f.m_sides[OppositeDirection(input2)].m_bAddPrefix = false;
          }
          f.m_sides[OppositeDirection(input2)].m_strength = 2;
          f.m_sides[OppositeDirection(input2)].m_label = ps.at(i).ToString();
        }
      }
    }

    if (!bOnAxis) {
      tsF1.push_back(f);
    }
  }
  ts += tsF1;

  //  Create the SEED tile
  Tile t("SEED");
  t.m_display = "SEED";

  t.m_sides[NORTH].m_strength  = 2;
  t.m_sides[NORTH].m_label     = GetPrefixFromNum(0, NORTH, &ps) + MOSTSIG + "0" + LEASTSIG;

  t.m_sides[EAST].m_strength  = 2;
  t.m_sides[EAST].m_label     = GetPrefixFromNum(0, EAST, &ps) + MOSTSIG + "0" + LEASTSIG;

  t.m_sides[SOUTH].m_strength = 0;
  t.m_sides[SOUTH].m_label    = NODISPLAY;

  t.m_sides[WEST].m_strength  = 0;
  t.m_sides[WEST].m_label     = MOSTSIG;

  ts.push_back(t);

  //  Create vertical and horizontal counters of each type > 0 (i.e. for each point not equal to [0,0])
  for (size_t i=0; i<ps.size(); i++) {
    if (ps.at(i) != Point(0,0)) {
      TileSet tsTransitionSpacers = CreateTransitionSpacerTileSet(c+1, i, &ps);
      ts += tsTransitionSpacers;
    }

    TileSet tsVertCounter = CreateCounterTileSet(c+1, i, VERTICAL, &ps);
    ts += tsVertCounter;

    TileSet tsHorzCounter = CreateCounterTileSet(c+1, i, HORIZONTAL, &ps);
    ts += tsHorzCounter;
  }

  *pTS += ts;

  for (size_t i=0; i<pTS->size(); i++) {
    //  Rotate tiles as necessary for the desired quadrant
    if (quadrant == QUAD2) {
      pTS->at(i).Rotate(90);
    } else if (quadrant == QUAD3) {
      pTS->at(i).Rotate(180);
    } else if (quadrant == QUAD4) {
      pTS->at(i).Rotate(270);
    }
  }

  return true;
}