/////////////////////////////////////////////////////////////////////////////
// Name:        tiles.cpp
// Purpose:     Classes and helper functions to work with tiletype definitions
// Author:      Matthew Patitz <mpatitz@yahoo.com>
// Modified by:
// Created:     Nov/2007
// Copyright:   (c) 2008, Matthew John Patitz
// Licence:     GPL
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "tiles.h"


std::string itos(int i)
{
  char buf[256] = {0};
  return std::string(_itoa(i, buf, 10));
}

void SplitString(std::string str, std::string delim, std::vector<std::string> &results)
{
  size_t cutAt;
  while((cutAt = str.find_first_of(delim)) != str.npos)
  {
    if(cutAt > 0) {
      results.push_back(str.substr(0,cutAt));
    }
    str = str.substr(cutAt+1);
  }
  if(str.length() > 0) {
    results.push_back(str);
  }
}

std::string StripSpaces(std::string& str)
{
  std::string out = str;
  while (out.length() > 0 && out.at(0) == ' ') {
    out = out.substr(1);
  }
  while (out.length() > 0 && out.at(out.length()-1) == ' ') {
    out = out.substr(0, out.length()-1);
  }
  return out;
}

int OppositeDirection(int dir)
{
  return ((dir + 2) % 4);
}

int DirectionIntFromString(std::string sDir)
{
  int ret = -1;
  if (sDir == "N" || sDir == "n") {
    ret = NORTH;
  } else if (sDir == "E" || sDir == "e") {
    ret = EAST;
  } else if (sDir == "S" || sDir == "s") {
    ret = SOUTH;
  } else if (sDir == "W" || sDir == "w") {
    ret = WEST;
  }
  return ret;
}

std::string DirectionStringFromInt(int iDir)
{
  if (iDir == NORTH) {
    return std::string("N");
  } else if (iDir == EAST) {
    return std::string("E");
  } else if (iDir == SOUTH) {
    return std::string("S");
  } else if (iDir == WEST) {
    return std::string("W");
  } else {
    return std::string("ERROR");
  }
}

int GetOrientationFromDirection(int dir)
{
  if ((dir == NORTH) || (dir == SOUTH)) {
    return VERTICAL;
  } else if ((dir == EAST) || (dir == WEST)) {
    return HORIZONTAL;
  }
  return -1;
}

int OppositeOrientation(int orient)
{
  if (orient == HORIZONTAL) {
    return VERTICAL;
  } else if (orient == VERTICAL) {
    return HORIZONTAL;
  }
  return -1;
}

std::vector<std::string> StringSet::StringSetHelper(std::vector<std::vector<std::string>>& vvs, size_t i)
{
  if (i == (vvs.size()-1)) {
    return vvs[i];
  }

  std::vector<std::string> retVS;
  std::vector<std::string>& vs = vvs[i];
  std::vector<std::string> subVS = StringSetHelper(vvs, i+1);

  for (size_t n=0; n<vs.size(); n++) {
    for (size_t m=0; m<subVS.size(); m++) {
      retVS.push_back(vs[n] + subVS[m]);
    }
  }
  return retVS;
}

//  %s - string
//  %i - int
//  %r - range of int's ("x,y")
//  %Sx - set of strings (x number of strings)
StringSet::StringSet(const char* fmt, ...)
{
  std::vector<std::vector<std::string>> preSSet;
  va_list ap;
  const char* p = fmt;
  va_start(ap, fmt);
  while (*p) {
    std::vector<std::string> v;
    if (strncmp(p, "%s", 2) == 0) {
      std::string s = va_arg(ap, char*);
      v.push_back(s);
      preSSet.push_back(v);
      p = p + 2;
    } else if (strncmp(p, "%i", 2) == 0) {
      int i = va_arg(ap, int);
      v.push_back(itos(i));
      preSSet.push_back(v);
      p = p + 2;
    } else if (strncmp(p, "%r", 2) == 0) {
      int i1 = va_arg(ap, int);
      int i2 = va_arg(ap, int);
      for (int i=i1; i<=i2; i++) {
        v.push_back(itos(i));
      }
      preSSet.push_back(v);
      p = p + 2;
    } else if (strncmp(p, "%S", 2) == 0) {
      p = p + 2;
      int n = atoi(p);
      for (int i=0; i<n; i++) {
        v.push_back(std::string(va_arg(ap, char*)));
      }
      preSSet.push_back(v);
      while (n > 0) {
        n = n - (n % 10);
        n = n / 10;
        p++;
      }
    } else {
      std::cout << "\nERROR:  Invalid StringSet construction!  Aborting.\n\n";
      exit(1);
    }
  }
  va_end(ap);

  for (size_t i=0; i<(preSSet[0].size()); i++) {
    if (preSSet.size() > 1) {
      std::vector<std::string> vs = StringSetHelper(preSSet, 1);
      for (size_t j=0; j<vs.size(); j++) {
        push_back(preSSet[0][i] + vs[j]);
      }
    } else {
      push_back(preSSet[0][i]);
    }
  }
}

Tile::Tile(const Tile& t)
{
  m_display = t.m_display;
  m_prefix  = t.m_prefix;
  m_suffix  = t.m_suffix;
  for (int i=NORTH; i<=WEST; i++) {
    m_sides[i] = t.m_sides[i];
  }
  m_name  = t.m_name;
  m_color = t.m_color;
}

void Tile::GenerateName()
{
  if (m_name.length() == 0) {
    m_name = m_sides[NORTH].m_label + NAMEDELIMIT + m_sides[EAST].m_label + NAMEDELIMIT + m_sides[SOUTH].m_label + NAMEDELIMIT + m_sides[WEST].m_label;
  }
}

void Tile::Print(std::ofstream* o)
{
  GenerateName();

  *o << "TILENAME ";
  *o << m_prefix.c_str();
  *o << m_name.c_str();
  *o << m_suffix.c_str();
  *o << "\n";

  if (m_color != "") {
    *o << "TILECOLOR ";
    *o << m_color;
    *o << "\n";
  }

  *o << "LABEL ";
  *o << m_prefix.c_str();
  *o << m_display.c_str();
  *o << m_suffix.c_str();
  *o << "\n";

  *o << "NORTHBIND ";
  *o << m_sides[NORTH].m_strength;
  *o << "\n";

  *o << "EASTBIND ";
  *o << m_sides[EAST].m_strength;
  *o << "\n";

  *o << "SOUTHBIND ";
  *o << m_sides[SOUTH].m_strength;
  *o << "\n";

  *o << "WESTBIND ";
  *o << m_sides[WEST].m_strength;
  *o << "\n";

  *o << "NORTHLABEL ";
  if (m_sides[NORTH].m_bAddPrefix && (m_sides[NORTH].m_label != NODISPLAY)) {
    *o << m_prefix.c_str();
  }
  *o << m_sides[NORTH].m_label;
  if (m_sides[NORTH].m_label != NODISPLAY) {
    *o << m_suffix.c_str();
  }
  *o << "\n";

  *o << "EASTLABEL ";
  if (m_sides[EAST].m_bAddPrefix && (m_sides[EAST].m_label != NODISPLAY)) {
    *o << m_prefix.c_str();
  }
  *o << m_sides[EAST].m_label;
  if (m_sides[EAST].m_label != NODISPLAY) {
    *o << m_suffix.c_str();
  }
  *o << "\n";

  *o << "SOUTHLABEL ";
  if (m_sides[SOUTH].m_bAddPrefix && (m_sides[SOUTH].m_label != NODISPLAY)) {
    *o << m_prefix.c_str();
  }
  *o << m_sides[SOUTH].m_label;
  if (m_sides[SOUTH].m_label != NODISPLAY) {
    *o << m_suffix.c_str();
  }
  *o << "\n";

  *o << "WESTLABEL ";
  if (m_sides[WEST].m_bAddPrefix && (m_sides[WEST].m_label != NODISPLAY)) {
    *o << m_prefix.c_str();
  }
  *o << m_sides[WEST].m_label;
  if (m_sides[WEST].m_label != NODISPLAY) {
    *o << m_suffix.c_str();
  }
  *o << "\n";

  *o << "CREATE\n";
}

//  Rotation is done in the clockwise direction
void Tile::Rotate(int degrees)
{
  if (degrees != 0 && degrees != 90 && degrees != 180 && degrees != 270 && degrees != 360)
  {
    std::cout << "Error:  Invalid input to Tile::Rotate - " + itos(degrees) + "\n";
    exit(1);
  }
  
  TileSides ts = m_sides;
  int rotCount = degrees / 90;
  for (int i=0; i<4; i++) {
    m_sides[(i+rotCount)%4] = ts[i];
  }
}

void Tile::Reflect(int axis)
{
  TileSides ts = m_sides;
  if (axis == HORIZONTAL) {
    m_sides[NORTH] = ts[SOUTH];
    m_sides[SOUTH] = ts[NORTH];
  } else if (axis == VERTICAL) {
    m_sides[EAST] = ts[WEST];
    m_sides[WEST] = ts[EAST];
  }
}

void Tile::UsePrefix(bool bUse)
{
  for (size_t i=0; i<m_sides.size(); i++) {
    m_sides[i].m_bAddPrefix = bUse;
  }
}

TileSides::TileSides()
{
  clear();
  push_back(TileSide());
  push_back(TileSide());
  push_back(TileSide());
  push_back(TileSide());
}

//  For any StringSet pointers that are equal, those strings will match for all tiles
TileSet::TileSet(StringSet* ssDisplay, int iNorthStrength, StringSet* ssNorthGlues, int iEastStrength, StringSet* ssEastGlues,
                                       int iSouthStrength, StringSet* ssSouthGlues, int iWestStrength, StringSet* ssWestGlues)
{
  for (size_t d=0; d<ssDisplay->size(); d++) {
    std::string display = ssDisplay->at(d);
    
    bool bNLoop = true;
    std::string nGlue;
    size_t nSize = ssNorthGlues->size();
    if (ssNorthGlues == ssDisplay) {
      nGlue = display;
      nSize = 1;
      bNLoop = false;
    }
    for (size_t n=0; n<nSize; n++) {
      if (bNLoop) {
        nGlue = ssNorthGlues->at(n);
      }

      bool bELoop = true;
      std::string eGlue;
      size_t eSize = ssEastGlues->size();
      if (ssEastGlues == ssDisplay) {
        eGlue = display;
        eSize = 1;
        bELoop = false;
      } else if (ssEastGlues == ssNorthGlues) {
        eGlue = nGlue;
        eSize = 1;
        bELoop = false;
      }
      for (size_t e=0; e<eSize; e++) {
        if (bELoop) {
          eGlue = ssEastGlues->at(e);
        }

        bool bSLoop = true;
        std::string sGlue;
        size_t sSize = ssSouthGlues->size();
        if (ssSouthGlues == ssDisplay) {
          sGlue = display;
          sSize = 1;
          bSLoop = false;
        } else if (ssSouthGlues == ssNorthGlues) {
          sGlue = nGlue;
          sSize = 1;
          bSLoop = false;
        } else if (ssSouthGlues == ssEastGlues) {
          sGlue = eGlue;
          sSize = 1;
          bSLoop = false;
        }
        for (size_t s=0; s<sSize; s++) {
          if (bSLoop) {
            sGlue = ssSouthGlues->at(s);
          }

          bool bWLoop = true;
          std::string wGlue;
          size_t wSize = ssWestGlues->size();
          if (ssWestGlues == ssDisplay) {
            wGlue = display;
            wSize = 1;
            bWLoop = false;
          } else if (ssWestGlues == ssNorthGlues) {
            wGlue = nGlue;
            wSize = 1;
            bWLoop = false;
          } else if (ssWestGlues == ssEastGlues) {
            wGlue = eGlue;
            wSize = 1;
            bWLoop = false;
          } else if (ssWestGlues == ssSouthGlues) {
            wGlue = sGlue;
            wSize = 1;
            bWLoop = false;
          }
          for (size_t w=0; w<wSize; w++) {
            if (bWLoop) {
              wGlue = ssWestGlues->at(w);
            }

            Tile t;
            t.m_display                 = display;
            t.m_sides[NORTH].m_strength = iNorthStrength;
            t.m_sides[EAST].m_strength  = iEastStrength;
            t.m_sides[SOUTH].m_strength = iSouthStrength;
            t.m_sides[WEST].m_strength  = iWestStrength;
            t.m_sides[NORTH].m_label    = nGlue;
            t.m_sides[EAST].m_label     = eGlue;
            t.m_sides[SOUTH].m_label    = sGlue;
            t.m_sides[WEST].m_label     = wGlue;
            push_back(t);
          }
        }
      }
    }
  }
}

TileSet& TileSet::operator+(const TileSet& ts)
{
  for (size_t i=0; i<ts.size(); i++) {
    push_back(ts[i]);
  }
  return *this;
}

TileSet& TileSet::operator+=(const TileSet& ts)
{
  *this = *this + ts;
  return *this;
}

void TileSet::SetPrefix(std::string sPrefix)
{
  for (size_t i=0; i<size(); i++) {
    at(i).m_prefix = sPrefix;
  }
}

void TileSet::SetSuffix(std::string sSuffix)
{
  for (size_t i=0; i<size(); i++) {
    at(i).m_suffix = sSuffix;
  }
}

void TileSet::SetColor(std::string color)
{
  for (size_t i=0; i<size(); i++) {
    at(i).m_color = color;
  }
}

Tile* TileSet::GetTile(std::string sName)
{
  for (size_t i=0; i<size(); i++) {
    if (sName == at(i).GetName()) {
      return &at(i);
    }
  }
  return NULL;
}

bool TileSet::PrintToFile(const char* fileName, int type)
{
  std::ofstream out(fileName);
  if (!out.is_open()) {
    return false;
  }

  for (size_t j=0; j<size(); j++) {
    if (type == TILESIM_TILETYPES) {
      at(j).m_color = "";
    }
    at(j).Print(&out);
    out << "\n";
  }
  out.close();

  return true;
}


